/*************************************************************************
*
* YPOPs!
*
* YPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ConfigEmailPage.cpp
*
* Purpose : Configuration Dialog - Email Page
*
* Authors : Anuj Seth
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifdef WIN32
#include "ypops.h"
#include "ConfigEmailPage.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern CDataHolder settings;

IMPLEMENT_DYNCREATE(CConfigEmailPage, CPropertyPage)

CConfigEmailPage::CConfigEmailPage() : CCustomPropertyPage(CConfigEmailPage::IDD)
{
	//{{AFX_DATA_INIT(CConfigEmailPage)
	m_nEmailCategory = -1;
	m_bEmptyTrash = FALSE;
	m_bEmptyBulkMail = FALSE;
	m_nMaxEmails = 10000;
	m_bApplyLimit = FALSE;
	m_bIgnoreSize = FALSE;
	m_bLeaveUnread = FALSE;
	//}}AFX_DATA_INIT
}

CConfigEmailPage::~CConfigEmailPage()
{
}

void CConfigEmailPage::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CConfigEmailPage)
	DDX_Radio(pDX, IDC_EMAIL_CATEGORY, m_nEmailCategory);
	DDX_Check(pDX, IDC_EMPTY_TRASH, m_bEmptyTrash);
	DDX_Check(pDX, IDC_EMPTY_BULK_MAIL, m_bEmptyBulkMail);
	DDX_Text(pDX, IDC_MAX_EMAILS, m_nMaxEmails);
	DDX_Check(pDX, IDC_IGNORE_SIZE, m_bIgnoreSize);
	DDX_Control(pDX, IDC_APPLY_LIMIT, m_ctlApplyLimit);
	DDX_Check(pDX, IDC_APPLY_LIMIT, m_bApplyLimit);
	DDV_MinMaxUInt(pDX, m_nMaxEmails, 1, 10000);
	DDX_Check(pDX, IDC_LEAVE_UNREAD, m_bLeaveUnread);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CConfigEmailPage, CCustomPropertyPage)
	//{{AFX_MSG_MAP(CConfigEmailPage)
	ON_BN_CLICKED(IDC_IGNORE_SIZE, OnIgnoreSize)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CConfigEmailPage::Init(CDataHolder &data)
{
	m_nEmailCategory = (data.m_nEmailCategory == EC_ALL ? 0 :
						(data.m_nEmailCategory == EC_UNREAD ? 1 : 2));
	m_bEmptyTrash    = data.m_bEmptyTrash;
	m_nMaxEmails     = data.m_nMaxEmails;
    m_bIgnoreSize	 = data.m_bIgnoreSize;
    m_bApplyLimit	 = data.m_bApplyLimit;
	m_ctlApplyLimit.EnableWindow(m_bIgnoreSize);
	m_bLeaveUnread	 = data.m_bLeaveUnread;
	m_bEmptyBulkMail = data.m_bEmptyBulkMail;

	UpdateData(FALSE);
}

BOOL CConfigEmailPage::ValidateData(void)
{
	UpdateData(TRUE);

/*	if(!m_bEnableBulkMail && !m_bEnableInbox)
	{
		::AfxMessageBox("Select Inbox and/or Bulk Mail download", MB_OK | MB_ICONSTOP);
		return FALSE;
	}
*/
	return TRUE;
}

void CConfigEmailPage::SaveData(CDataHolder &data)
{
	settings.m_nEmailCategory= m_nEmailCategory == 0 ? EC_ALL :
							   (m_nEmailCategory == 1 ? EC_UNREAD : EC_FLAGGED);
	settings.m_bEmptyTrash	 = m_bEmptyTrash;
	settings.m_nMaxEmails	 = m_nMaxEmails;
	settings.m_bLeaveUnread	 = m_bLeaveUnread;
	settings.m_bEmptyBulkMail= m_bEmptyBulkMail;
    settings.m_bIgnoreSize = m_bIgnoreSize;
    settings.m_bApplyLimit = m_bApplyLimit;

	data.m_nEmailCategory	= m_nEmailCategory == 0 ? EC_ALL :
							   (m_nEmailCategory == 1 ? EC_UNREAD : EC_FLAGGED);
	data.m_bEmptyTrash		= m_bEmptyTrash;
	data.m_nMaxEmails		= m_nMaxEmails;
	data.m_bLeaveUnread		= m_bLeaveUnread;
	data.m_bEmptyBulkMail   = m_bEmptyBulkMail;
    data.m_bIgnoreSize = m_bIgnoreSize;
    data.m_bApplyLimit = m_bApplyLimit;
}

void CConfigEmailPage::OnIgnoreSize() 
{
    UpdateData(TRUE);
	if (!m_bIgnoreSize) {
		m_ctlApplyLimit.SetCheck(BST_CHECKED);
		m_bApplyLimit=TRUE;
	}
	m_ctlApplyLimit.EnableWindow(m_bIgnoreSize);
}

#endif
