/*************************************************************************
*
* YPOPs!
*
* YPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	DataHolder.cpp
*
* Purpose : Retrieve/Save the configured settings
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"

#ifndef WIN32
#include <iostream>
#include <errno.h>
#include <stdlib.h>
#include <string.h>
#endif

#include "DataHolder.h"
#include "LogFile.h"
#include "YPOPs.h"
#include <sys/stat.h> // for _stat()

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

#ifdef WIN32
extern CSystemTrayApp theApp;
#endif // WIN32
extern LogFile logFile;

CIniFile inf;

/*******************************************************************
* Function  : CDataHolder constructor
* Parameters: 
* Returns   :
* Purpose   : Initialize the object with defaults
*******************************************************************/
CDataHolder::CDataHolder()
{
	inipath = "./ypops.ini";
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   :
* Purpose   :
*******************************************************************/
CDataHolder::~CDataHolder()
{
}

#ifdef WIN32
/////////////////////////////////////////////////////////////////////////////////////////////////
//		GetShellFolderPath - Function to Get the Path to the Shell Folders 					   //
//																							   //
//		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpShellFolder													   //
//					: should contain one of the following,									   //
//						AppData, Cache, Cookies, Desktop, Favorites, Fonts, History, NetHood,  //
//						Personal, Printhood, Programs, Recent, SendTo, Start Menu, Startup,	   //
//						Templates, ShellNew													   //
//																							   //
//					: LPSTR lpShellPath														   //
//						will be populated with full path of the desired Shell Folder.		   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool GetShellFolderPath(LPCSTR lpShellFolder, LPSTR lpShellPath)
{
	//lpShellFolder can be 
	//AppData,Cache,Cookies,Desktop,Favorites,Fonts,History,NetHood,
	//Personal,Printhood,Programs,Recent,SendTo,Start Menu,Startup,
	//Templates, ShellNew
	DWORD rc;
	DWORD length = MAX_PATH;
	DWORD type = REG_SZ;
    HKEY hkey;
	
	rc = RegOpenKeyEx(HKEY_CURRENT_USER,
			"Software\\Microsoft\\Windows\\CurrentVersion\\Explorer\\Shell Folders",
			0, KEY_READ, &hkey);
	
	if (rc == ERROR_SUCCESS)
	{
		rc = RegQueryValueEx(hkey, lpShellFolder, NULL, &type,
			(BYTE *) lpShellPath, &length);
		RegCloseKey(hkey);
	}
	
	if (rc == ERROR_SUCCESS)
		return true;
	else
		return false;
}

/////////////////////////////////////////////////////////////////////////////////////////////////
//		CreateShortCut - Function to Create Short Cut					 					   //
//																							   //
//		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpszSourceFile												   //
//					: should contain the Source file for which the shourtcut has to be created.//
//																							   //
//					: LPCSTR lpszDestination												   //
//						Should contain the target directory and the shourt cut file name.	   //
//																							   //
//					: LPCSTR lpszDesc														   //
//						can contain any suitable description.								   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool CreateShortCut(LPCSTR lpszSourceFile,
					LPCSTR lpszDestination, 
					LPCSTR lpszDesc,
					LPCSTR lpszDirectory) 
{ 
    HRESULT		hResult    = NULL; 
    IShellLink* pShellLink = NULL; 
	bool		rc		   = false;
	
	hResult = CoInitialize(NULL);
    // Initialize the IShellLink interface. 
    
	hResult = CoCreateInstance(CLSID_ShellLink, NULL, 
        CLSCTX_INPROC_SERVER, IID_IShellLink, (void**) &pShellLink); 
	// Get a pointer to the IShellLink Interface
	
    if (SUCCEEDED(hResult)) 
	{ 
        IPersistFile* ppf = NULL; 
		
        pShellLink->SetPath(lpszSourceFile); 
        // Set the path to the shortcut target

		pShellLink->SetWorkingDirectory(lpszDirectory);
        
		hResult = pShellLink->SetDescription(lpszDesc); 
		// Add Description
		
        hResult = pShellLink->QueryInterface(IID_IPersistFile, (void**) &ppf); 
		// Query IShellLink for the IPersistFile interface for saving the 
        // shortcut in persistent storage. 
		
        if (SUCCEEDED(hResult)) 
		{ 
            WORD wszWideString[MAX_PATH]; 
			
            // Ensure that the string is ANSI. 
            MultiByteToWideChar(CP_ACP, 0, lpszDestination, -1,
				wszWideString, MAX_PATH); 
			
            hResult = ppf->Save(wszWideString, TRUE); 
            // Save the link by calling IPersistFile::Save. 
            
			ppf->Release(); 
			rc = true;
        } 
        pShellLink->Release(); 
		CoUninitialize();
		// Clean up
    } 
	return rc; 
} 

/////////////////////////////////////////////////////////////////////////////////////////////////
//		LaunchAtStartup - Function to Launch a File or Program during Startup				   //
//					(by creating a short cut in the startup folder)							   //
//																							   ////		Return Type : bool																	   //
//					: Returns true if successfull or false if errored.						   //
//																							   //
//		Parameters  : LPCTSTR lpszFileName													   //
//					: should contain the Full File Name for your Application.				   //
/////////////////////////////////////////////////////////////////////////////////////////////////
bool LaunchAtStartup(LPCTSTR lpszFileName, 
					 LPCTSTR lpszDescription,
					 LPCTSTR lpszDirectory)
{
	char szDestFileName[MAX_PATH] = "";
	char cBackwardSlash			= '\\';
	
	if (lpszFileName == NULL)
		return false;
	// File Name cannot be empty
	
	if(! GetShellFolderPath("Startup",szDestFileName))
		return false;
	// Error getting Startup Shell Folder Path
	
	strcat(szDestFileName,strrchr(lpszFileName,cBackwardSlash));
	strcat(szDestFileName,".lnk");
	
	return CreateShortCut(lpszFileName, szDestFileName, lpszDescription, lpszDirectory);
	// Create a Short up for the file or application in the start up so that 
	// the file or application starts up during Windows Startup.
}

#endif // WIN32

/*******************************************************************
* Function  : Save()
* Parameters: -
* Returns   : -
* Purpose   : To save the configured settings to the registry
*******************************************************************/
void CDataHolder::Save(const char *user_a)
{
	if (!user_a)
		SaveSection("global");
	else
		SaveSection(user_a);

	inf.WriteIniFile(inipath);
}

/*******************************************************************
* Function  : Load()
* Parameters: -
* Returns   : -
* Purpose   : To load the configured settings from the registry
*******************************************************************/
CStringEx CDataHolder::stripQ(const char *strA)
{
	CStringEx str = strA;
	if (*strA == '\"')
		str = CStringEx(str.substr(1, str.length()-2));
        return str;
}

int CDataHolder::loadOld()
{
#ifdef WIN32
	m_bSawDisclaimer			= theApp.GetProfileInt("Settings", "DisclaimerRead0.5.0", DEF_SAWDISCLAIMER);

#ifdef WIN32
	// determine m_bAutoStartWithWindows
	{
		char shortcut[MAX_PATH] = "";
		struct stat buf;

		/* Remove shortcut */
		if(GetShellFolderPath("Startup", shortcut)) {
			strcat(shortcut, "\\YPOPs.lnk");
            if (stat(shortcut, &buf) == -1)
				m_bAutoStartWithWindows = FALSE;
			else
				m_bAutoStartWithWindows = TRUE;
		}
	}
#endif

    m_nHttpProxyPort	= theApp.GetProfileInt("Settings", "HttpProxyPort", DEF_HTTPPROXYPORT);
	m_nHttpsProxyPort	= theApp.GetProfileInt("Settings", "HttpsProxyPort", DEF_HTTPSPROXYPORT);
	m_bUseProxy			= theApp.GetProfileInt("Settings", "UseProxy", DEF_USEPROXY);
	m_strHttpProxyHost	= CStringEx(theApp.GetProfileString("Settings", "HttpProxyHost", NULL));
	m_strHttpsProxyHost	= CStringEx(theApp.GetProfileString("Settings", "HttpsProxyHost", NULL));
	m_UserAgent			= CStringEx(theApp.GetProfileString("Settings", "UserAgent", DEF_USERAGENT));
	m_nMaxEmails		= theApp.GetProfileInt("Settings", "MaxEmails", DEF_MAXEMAILS);
	m_bEmptyTrash		= theApp.GetProfileInt("Settings", "EmptyTrash", DEF_EMPTYTRASH);
	m_bEmptyBulkMail	= theApp.GetProfileInt("Settings", "EmptyBulkMail", DEF_EMPTYBULKMAIL);
	m_bBulkMail			= theApp.GetProfileInt("Settings", "BulkMail", DEF_BULKMAIL);
	m_bInbox			= theApp.GetProfileInt("Settings", "Inbox", DEF_INBOX);
	m_nEmailCategory	= (EMAIL_CATEGORY)theApp.GetProfileInt("Settings", "EmailCategory", DEF_EMAILCATEGORY);
	m_bProxyAuth		= theApp.GetProfileInt("Settings", "ProxyAuth", DEF_PROXYAUTH);
	m_strProxyAuthPassword = CStringEx(theApp.GetProfileString("Settings", "ProxyAuthPassword", ""));
	m_strProxyAuthUsername = CStringEx(theApp.GetProfileString("Settings", "ProxyAuthUsername", ""));
	m_bQuietMode		= theApp.GetProfileInt("Settings", "QuietMode", DEF_QUIETMODE);
	m_strNetworkIPAddr  = CStringEx(theApp.GetProfileString("Settings", "NetworkIP", DEF_NETWORKIP));
	m_nNetworkPort		= theApp.GetProfileInt("Settings", "NetworkPort", DEF_POP3PORT);
	m_bLeaveUnread		= theApp.GetProfileInt("Settings", "LeaveUnread", DEF_LEAVEUNREAD);
	m_nSmtpPort			= theApp.GetProfileInt("Settings", "SmtpPort", DEF_SMTPPORT);
	m_strFolderList     = CStringEx(theApp.GetProfileString("Settings", "Folders", ""));
    m_bAlwaysOnTop      = theApp.GetProfileInt("Settings", "AlwaysOnTop", DEF_ALWAYSONTOP);
    m_bEnableSmtp       = theApp.GetProfileInt("Settings", "EnableSmtp", DEF_ENABLESMTP);
    m_bSaveEmailCopy    = theApp.GetProfileInt("Settings", "SaveEmailCopy", DEF_SAVEEMAILCOPY);
    m_bEnableUIDL       = theApp.GetProfileInt("Settings", "EnableUIDL", DEF_ENABLEUIDL);
    m_bIgnoreSize       = theApp.GetProfileInt("Settings", "IgnoreExactSize", DEF_IGNOREEXACTSIZE);
    m_bApplyLimit       = theApp.GetProfileInt("Settings", "ApplyLimitToList", DEF_APPLYLIMITTOLIST);
    m_bHideTrayIcon     = theApp.GetProfileInt("Settings", "HideTrayIcon", DEF_HIDETRAYICON);
	m_nCookieTimeout    = theApp.GetProfileInt("Settings", "CookieTimeout", DEF_COOKIETIMEOUT);

	// Activity Log
	m_bCreateLog		= theApp.GetProfileInt("Settings", "CreateLog", DEF_CREATELOG);
	m_bLimitLog			= theApp.GetProfileInt("Settings", "LimitLog", DEF_LIMITLOG);
	m_nLogLevel			= theApp.GetProfileInt("Settings", "LogLevel", DEF_LOGLEVEL);
	m_nLogSize			= theApp.GetProfileInt("Settings", "LogSize", DEF_LOGSIZE);


    return 1;
#else
	/* the define CONFIG_FILE_LOC comes from the makefile which inturn comes from the user :-) */
	CStringEx filename;
	filename.Format("%s/ypopsrc", confdirenv ? confdirenv : DEFCONFIGDIR);
	FILE *fp=NULL;
	char temp_str[1024];
	if ( ( fp = fopen(filename,"r") ) == NULL )
	{
		fprintf(stderr,"The necessary preferences file was not found.  Please copy the sample preferences file ypops_samplerc given along with the distribution to %s\n",filename.c_str());
		return(0);
	}
	/* first the inbox setting */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bInbox = atoi(temp_str);
	/* now the bulk e-mail setting*/
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bBulkMail = atoi(temp_str);
	/* emptying trash on exit */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bEmptyTrash = atoi(temp_str);
	/* leave messages unread */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bLeaveUnread = atoi(temp_str);
	/*maximum number of e-mails */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nMaxEmails = atoi(temp_str);
	/*downloading all e-mails */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	if ( ( strcmp(temp_str,"a") ) == 0 )
	{
		m_nEmailCategory = EC_ALL;
	}
	else
	if ( ( strcmp(temp_str,"u") ) == 0 )
	{
		m_nEmailCategory = EC_UNREAD;
	}
	else
	if ( ( strcmp(temp_str,"f") ) == 0 )
	{
		m_nEmailCategory = EC_FLAGGED;
	}
	/* connection on secure mode */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	//m_bSecureMode = atoi(temp_str);
	/*password using MD5 */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	//m_bUseMD5 = atoi(temp_str);
	/*using proxy server ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bUseProxy = atoi(temp_str);
	/*http proxy server address */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strHttpProxyHost = stripQ(temp_str);
	/*http proxy server port number */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nHttpProxyPort = atoi(temp_str);
	/*secure proxy server address */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strHttpsProxyHost = stripQ(temp_str);
	/* secure proxy server port number */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nHttpsProxyPort = atoi(temp_str);
	/* use proxy authentication ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bProxyAuth = atoi(temp_str);
	/* proxy server username */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strProxyAuthUsername = stripQ(temp_str);
	/* proxy server password */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strProxyAuthPassword = stripQ(temp_str);
	/* the port we must run on */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nNetworkPort = atoi(temp_str);
	/* do we need log file generation ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bCreateLog = atoi(temp_str);
	/* basic logs ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogLevel = atoi(temp_str);
	/* advanced logs ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogLevel = atoi(temp_str);
	/* do we need to limit log file size ? */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bLimitLog = atoi(temp_str);
	/* maximum log file size */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nLogSize = atoi(temp_str);
	/* the smtp port where we have to listen for connections */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nSmtpPort = atoi(temp_str);
	/* whether we need smtp or not */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bEnableSmtp = atoi(temp_str);
	/* whether to keep mail in yahoo sent mail folder */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_bSaveEmailCopy = atoi(temp_str);
	/* whether to enable UIDL */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
 	m_bEnableUIDL = atoi(temp_str);
	/* the list of all folders seperated by a pipe | */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strFolderList = stripQ(temp_str);

	/* Cookie timeout */
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_nCookieTimeout = atoi(temp_str);

	// m_strNetworkIPAddr
	fscanf(fp,"%s",temp_str);
	fscanf(fp,"%s",temp_str);
	m_strNetworkIPAddr = stripQ(temp_str);

	Save(); // no UI in unix, so we need to save all settings to new .ini file

	/* i think we are done for now.. 
	 * so return ! */
	return(1);
#endif
}

int CDataHolder::Load(const char *user_a)
{
	errno = 0;
	if (!inf.OpenIniFile(inipath)) {
		if (errno == ENOENT) {
			loadOld(); // convert old format to new and save .ini file
			Save(user_a);
			if (!inf.OpenIniFile(inipath)) {
				logFile.Write(LOG_BASIC, "Could not open %s: %s", inipath, strerror(errno));
				logFile.Write(LOG_BASIC, "Aborting");
				exit(1);
			}
		} else {
			logFile.Write(LOG_BASIC, "Could not open %s: %s", inipath, strerror(errno));
			logFile.Write(LOG_BASIC, "Aborting");
			exit(1);
		}
	}

	LoadSection("global"); // load defaults

#ifdef WIN32
	// determine m_bAutoStartWithWindows
	{
		char shortcut[MAX_PATH] = "";
		struct stat buf;

		/* Remove shortcut */
		if(GetShellFolderPath("Startup", shortcut)) {
			strcat(shortcut, "\\YPOPs.lnk");
            if (stat(shortcut, &buf) == -1)
				m_bAutoStartWithWindows = FALSE;
			else
				m_bAutoStartWithWindows = TRUE;
		}
	}
#endif

	if (user_a)
		LoadSection(user_a) ; // load user-specific values

	return TRUE;
}

void CDataHolder::ReadString(CStringEx *svar, CCHR *pSection, CCHR *pKey, char *Default)
{
	*svar = inf.ReadString(pSection, pKey, !strcmp(pSection, "global") ? Default : *svar);
}

void CDataHolder::ReadStr(CStringEx *svar, CCHR *pSection, CCHR *pKey, char *Default)
{
	*svar = inf.ReadStr(pSection, pKey, !strcmp(pSection, "global") ? Default : *svar);
}

void CDataHolder::ReadBool(BOOL *var, CCHR *pSection, CCHR *pKey, int Default)
{
	*var = inf.ReadInt(pSection, pKey, !strcmp(pSection, "global") ? Default : *var);
}

void CDataHolder::ReadInt(UINT *var, CCHR *pSection, CCHR *pKey, int Default)
{
	*var = inf.ReadInt(pSection, pKey, !strcmp(pSection, "global") ? Default : *var);
}

void CDataHolder::LoadSection(const char *Section)
{
	ReadBool(&m_bSawDisclaimer, Section, "SawDisclaimer",			DEF_SAWDISCLAIMER);

	ReadInt(&m_nHttpProxyPort,	Section, "HttpProxyPort",			DEF_HTTPPROXYPORT);
	ReadInt(&m_nHttpsProxyPort, Section, "HttpsProxyPort",			DEF_HTTPSPROXYPORT);
	ReadBool(&m_bUseProxy,		Section, "UseProxy",				DEF_USEPROXY);
	ReadStr(&m_strHttpProxyHost,Section, "HttpProxyHost",			NULL);
	ReadStr(&m_strHttpsProxyHost,Section,"HttpsProxyHost",			NULL);
	ReadString(&m_UserAgent,	Section, "UserAgent",				DEF_USERAGENT);
	ReadString(&m_sYahooDomain,	Section, "YahooDomain",				DEF_YAHOODOMAIN);
	ReadInt(&m_nMaxEmails,		Section, "MaxEmails",				DEF_MAXEMAILS);
	ReadBool(&m_bEmptyTrash,	Section, "EmptyTrash",				DEF_EMPTYTRASH);
	ReadBool(&m_bEmptyBulkMail,	Section, "EmptyBulkMail",			DEF_EMPTYBULKMAIL);
	ReadBool(&m_bBulkMail,		Section, "BulkMail",				DEF_BULKMAIL);
	ReadBool(&m_bInbox,			Section, "Inbox",					DEF_INBOX);
	UINT uiEmailCategory;
	ReadInt(&uiEmailCategory,	Section, "EmailCategory",			DEF_EMAILCATEGORY);
	m_nEmailCategory	= (EMAIL_CATEGORY)uiEmailCategory;
	ReadBool(&m_bProxyAuth,		Section, "ProxyAuth",				DEF_PROXYAUTH);
	ReadStr(&m_strProxyAuthPassword,Section, "ProxyAuthPassword",	"");
	ReadStr(&m_strProxyAuthUsername,Section, "ProxyAuthUsername",	"");
	ReadBool(&m_bQuietMode,		Section, "QuietMode",				DEF_QUIETMODE);
	ReadStr(&m_strNetworkIPAddr,Section, "NetworkIP",				DEF_NETWORKIP);
	ReadInt(&m_nNetworkPort,	Section, "Pop3Port",				DEF_POP3PORT);
	ReadBool(&m_bLeaveUnread,	Section, "LeaveUnread",				DEF_LEAVEUNREAD);
	ReadInt(&m_nSmtpPort,		Section, "SmtpPort",				DEF_SMTPPORT);
	ReadStr(&m_strFolderList,	Section, "Folders",					"");
	ReadBool(&m_bAlwaysOnTop,	Section, "AlwaysOnTop",				DEF_ALWAYSONTOP);
	ReadBool(&m_bEnableSmtp,	Section, "EnableSmtp",				DEF_ENABLESMTP);
	ReadBool(&m_bSaveEmailCopy,	Section, "SaveEmailCopy",			DEF_SAVEEMAILCOPY);
	ReadBool(&m_bEnableUIDL,	Section, "EnableUIDL",				DEF_ENABLEUIDL);
	ReadBool(&m_bIgnoreSize,	Section, "IgnoreExactSize",			DEF_IGNOREEXACTSIZE);
	ReadBool(&m_bApplyLimit,	Section, "ApplyLimitToList",		DEF_APPLYLIMITTOLIST);
	ReadBool(&m_bHideTrayIcon,	Section, "HideTrayIcon",			DEF_HIDETRAYICON);
	ReadInt(&m_nCookieTimeout,	Section, "CookieTimeout",			DEF_COOKIETIMEOUT);

	// Activity Log
	ReadBool(&m_bCreateLog,		Section, "CreateLog",				DEF_CREATELOG);
	ReadBool(&m_bLimitLog,		Section, "LimitLog",				DEF_LIMITLOG);
	ReadInt(&m_nLogLevel,		Section, "LogLevel",				DEF_LOGLEVEL);
	ReadInt(&m_nLogSize,		Section, "LogSize",					DEF_LOGSIZE);

	if (!m_bIgnoreSize) m_bApplyLimit = TRUE;

	return ;
}

/*=========================================================================
   WriteString : Writes a string to the ini file
*========================================================================*/
void CDataHolder::WriteStr(CCHR *pSection, CCHR *pKey, CCHR *pValue, CCHR *pDefault)
{
#ifdef WIN32
	BOOL WriteAll = FALSE;
#else
	BOOL WriteAll = TRUE;
#endif

	if (WriteAll || strcmp(pValue, pDefault))
		inf.WriteString(pSection, pKey, pValue);
	else
		inf.DeleteKey(pSection, pKey);
}

/*=========================================================================
   WriteBool : Writes a boolean to the ini file
*========================================================================*/
void CDataHolder::WriteBool (CCHR *pSection, CCHR *pKey, UINT Value, UINT Default)
{
	WriteStr(pSection, pKey, Value ? "1" : "0", Default ? "1" : "0");
}

/*=========================================================================
   WriteInt : Writes an integer to the ini file
*========================================================================*/
void CDataHolder::WriteInt (CCHR *pSection, CCHR *pKey, int Value, int Default)
{
	CStringEx Val, Def; /* 32bit maximum + sign + \0 */
	Val.Format("%d", Value);
	Def.Format("%d", Default);
	WriteStr(pSection, pKey, Val, Def);
}

void CDataHolder::SaveSection(const char *Section)
{
	if (!strcmp(Section, "global")) {
		// only the global section has these...
		inf.WriteBool(Section, "SawDisclaimer", m_bSawDisclaimer);

#ifdef WIN32
		if(m_bAutoStartWithWindows)
		{
			/* Create shortcut */
			char ypopsPath[MAX_PATH];	
			char ypops[MAX_PATH];

			GetCurrentDirectory(sizeof(ypopsPath), ypopsPath);
			strcpy(ypops, ypopsPath);
			strcat(ypops, "\\YPOPs");
			LaunchAtStartup(ypops, 
							"Launch YPOPs! at Windows startup",
							ypopsPath);
		}
		else
		{
			char shortcut[MAX_PATH] = "";

			/* Remove shortcut */
			if(GetShellFolderPath("Startup", shortcut))
			{
				strcat(shortcut, "\\YPOPs.lnk");
				unlink(shortcut);
			}
		}
#endif // WIN32
	
		WriteInt(Section,	"HttpProxyPort",	m_nHttpProxyPort,	DEF_HTTPPROXYPORT);
		WriteInt(Section,	"HttpsProxyPort",	m_nHttpsProxyPort,	DEF_HTTPSPROXYPORT);
		WriteBool(Section,	"UseProxy",			m_bUseProxy,		DEF_USEPROXY);
		WriteStr(Section,   "HttpProxyHost",	m_strHttpProxyHost, "");
		WriteStr(Section,   "HttpsProxyHost",	m_strHttpsProxyHost,"");
        CStringEx UserAgent = m_UserAgent+";";
        CStringEx DefAgent = DEF_USERAGENT;
        DefAgent += ";";
		WriteStr(Section,   "UserAgent",		UserAgent,	        DefAgent);
        CStringEx YahooDomain = m_sYahooDomain+";";
        CStringEx DefDomain = DEF_YAHOODOMAIN;
        DefDomain += ";";
		WriteStr(Section,   "YahooDomain",		m_sYahooDomain,	    DefDomain);
		WriteStr(Section,   "NetworkIP",		m_strNetworkIPAddr, DEF_NETWORKIP);
		WriteInt(Section,	"Pop3Port",			m_nNetworkPort,		DEF_POP3PORT);
		WriteBool(Section,	"HideTrayIcon",		m_bHideTrayIcon,	DEF_HIDETRAYICON);
		WriteInt(Section,	"CookieTimeout",	m_nCookieTimeout,	DEF_COOKIETIMEOUT);
		WriteInt(Section,	"SmtpPort",			m_nSmtpPort,		DEF_SMTPPORT);
		WriteBool(Section,	"AlwaysOnTop",		m_bAlwaysOnTop,		DEF_ALWAYSONTOP);
		WriteBool(Section,	"EnableSmtp",		m_bEnableSmtp,		DEF_ENABLESMTP);
		WriteBool(Section,	"CreateLog",		m_bCreateLog,		DEF_CREATELOG);
		WriteBool(Section,	"LimitLog",			m_bLimitLog,		DEF_LIMITLOG);
		WriteInt(Section,	"LogLevel",			m_nLogLevel,		DEF_LOGLEVEL);
		WriteInt(Section,	"LogSize",			m_nLogSize,			DEF_LOGSIZE);
	}

	// all sections have these.  global has the defaults for all users
		
	WriteInt(Section,   "MaxEmails",        m_nMaxEmails,           DEF_MAXEMAILS);
	WriteBool(Section,  "EmptyTrash",       m_bEmptyTrash,          DEF_EMPTYTRASH);
	WriteBool(Section,  "EmptyBulkMail",    m_bEmptyBulkMail,       DEF_EMPTYBULKMAIL);
	WriteBool(Section,  "BulkMail",         m_bBulkMail,            DEF_BULKMAIL);
	WriteBool(Section,  "Inbox",            m_bInbox,               DEF_INBOX);
	WriteInt(Section,   "EmailCategory",    m_nEmailCategory,       DEF_EMAILCATEGORY);
	WriteBool(Section,  "ProxyAuth",        m_bProxyAuth,           DEF_PROXYAUTH);
	WriteStr(Section,   "ProxyAuthPassword",m_strProxyAuthPassword, "");
	WriteStr(Section,   "ProxyAuthUsername",m_strProxyAuthUsername, "");
	WriteBool(Section,  "QuietMode",        m_bQuietMode,           DEF_QUIETMODE);
	WriteBool(Section,  "LeaveUnread",      m_bLeaveUnread,         DEF_LEAVEUNREAD);
	WriteStr(Section,   "Folders",          m_strFolderList,        "");
	WriteBool(Section,  "SaveEmailCopy",    m_bSaveEmailCopy,       DEF_SAVEEMAILCOPY);
	WriteBool(Section,  "EnableUIDL",       m_bEnableUIDL,          DEF_ENABLEUIDL);
	WriteBool(Section,  "IgnoreExactSize",  m_bIgnoreSize,          DEF_IGNOREEXACTSIZE);
	WriteBool(Section,  "ApplyLimitToList", m_bApplyLimit,          DEF_APPLYLIMITTOLIST);
}

/*******************************************************************
* Function  : DumpToLog()
* Parameters: -
* Returns   : -
* Purpose   : To write the configuration details to log file
*******************************************************************/
void CDataHolder::DumpToLog()
{
    logFile.Write(LOG_ADVANCED, "=========================================================================");
    logFile.Write(LOG_ADVANCED, MY_VERSION_DESC, MY_VERSION);
    logFile.Write(LOG_ADVANCED, "Configuration:");
    logFile.Write(LOG_ADVANCED, " Log Mode = %s", m_nLogLevel == 0 ? "Basic" : "Advanced");
//    logFile.Write(LOG_ADVANCED, " MD5 = %d, Secure Mode = %d", m_bUseMD5, m_bSecureMode);

    logFile.Write(LOG_ADVANCED, " Use Proxy = %d, HTTP Proxy = %s:%d, HTTPS Proxy = %s:%d", m_bUseProxy, (m_strHttpProxyHost.IsEmpty() ? "-" : LPCTSTR(m_strHttpProxyHost)),  m_nHttpProxyPort, (m_strHttpsProxyHost.IsEmpty() ? "-": LPCTSTR(m_strHttpsProxyHost)), m_nHttpsProxyPort); 
    logFile.Write(LOG_ADVANCED, " Max Emails = %d, Leave as unread = %d", m_nMaxEmails, m_bLeaveUnread);
    logFile.Write(LOG_ADVANCED, " Empty Trash = %d, Empty Bulk = %d", m_bEmptyTrash, m_bEmptyBulkMail);
    logFile.Write(LOG_ADVANCED, " Download Inbox = %d, Download BulkMail = %d", m_bInbox, m_bBulkMail);
    
    logFile.Write(LOG_ADVANCED, " Email Category = %s", 
        m_nEmailCategory == EC_ALL ? "All" : 
        m_nEmailCategory == EC_UNREAD ? "Unread" : "Flagged");

    logFile.Write(LOG_ADVANCED, " Proxy Auth = %d, Username = %s, Password = <HIDDEN>", m_bProxyAuth, (m_strProxyAuthUsername.IsEmpty() ? "-" : LPCTSTR(m_strProxyAuthUsername)));
    
    logFile.Write(LOG_ADVANCED, " Quiet Mode = %d, Autostart = %d, Always on Top = %d", m_bQuietMode, m_bAutoStartWithWindows, m_bAlwaysOnTop);
    logFile.Write(LOG_ADVANCED, " Network IP = %s, POP3 Port = %d, SMTP Enable = %d, SMTP Port = %d", LPCTSTR(m_strNetworkIPAddr), m_nNetworkPort, m_bEnableSmtp, m_nSmtpPort);
    logFile.Write(LOG_ADVANCED, " Save Sent Email = %d, Enable UIDL = %d, Ignore Exact Size = %d, Hide Tray Icon = %d", m_bSaveEmailCopy, m_bEnableUIDL, m_bIgnoreSize, m_bHideTrayIcon);
	logFile.Write(LOG_ADVANCED, " Apply Limit to Message List = %d, Cookie Timeout = %d", m_bApplyLimit, m_nCookieTimeout);
    logFile.Write(LOG_ADVANCED, " User Agent = %s", LPCTSTR(m_UserAgent));
    logFile.Write(LOG_ADVANCED, "=========================================================================");
}


#ifdef _USE_THIS_OLD_CODE

#ifdef WIN32
	theApp.WriteProfileInt("Settings", "UseMD5", m_bUseMD5);
	theApp.WriteProfileInt("Settings", "SecureMode", m_bSecureMode);
	theApp.WriteProfileInt("Settings", "PlainMode", m_bPlain);
	theApp.WriteProfileInt("Settings", "HttpProxyPort", m_nHttpProxyPort);
	theApp.WriteProfileInt("Settings", "HttpsProxyPort", m_nHttpsProxyPort);
	theApp.WriteProfileInt("Settings", "UseProxy", m_bUseProxy);
	theApp.WriteProfileString("Settings", "HttpProxyHost", m_strHttpProxyHost);
	theApp.WriteProfileString("Settings", "HttpsProxyHost", m_strHttpsProxyHost);
	theApp.WriteProfileString("Settings", "UserAgent", m_UserAgent);
	theApp.WriteProfileInt("Settings", "MaxEmails", m_nMaxEmails);
	theApp.WriteProfileInt("Settings", "EmptyTrash", m_bEmptyTrash);
	theApp.WriteProfileInt("Settings", "EmptyBulkMail", m_bEmptyBulkMail);
	theApp.WriteProfileInt("Settings", "BulkMail", m_bBulkMail);
	theApp.WriteProfileInt("Settings", "Inbox", m_bInbox);
	theApp.WriteProfileInt("Settings", "EmailCategory", m_nEmailCategory);
	theApp.WriteProfileInt("Settings", "ProxyAuth", m_bProxyAuth);
	theApp.WriteProfileString("Settings", "ProxyAuthPassword", m_strProxyAuthPassword);
	theApp.WriteProfileString("Settings", "ProxyAuthUsername", m_strProxyAuthUsername);
	theApp.WriteProfileInt("Settings", "QuietMode", m_bQuietMode);
	theApp.WriteProfileString("Settings", "NetworkIP", m_strNetworkIPAddr);
	theApp.WriteProfileInt("Settings", "NetworkPort", m_nNetworkPort);
	theApp.WriteProfileInt("Settings", "LeaveUnread", m_bLeaveUnread);
	theApp.WriteProfileInt("Settings", "SmtpPort", m_nSmtpPort);
	theApp.WriteProfileString("Settings", "Folders", m_strFolderList);
    theApp.WriteProfileInt("Settings", "AlwaysOnTop", m_bAlwaysOnTop);
    theApp.WriteProfileInt("Settings", "EnableSmtp", m_bEnableSmtp);
    theApp.WriteProfileInt("Settings", "SaveEmailCopy", m_bSaveEmailCopy);
    theApp.WriteProfileInt("Settings", "EnableUIDL", m_bEnableUIDL);
    theApp.WriteProfileInt("Settings", "IgnoreExactSize", m_bIgnoreSize);
    theApp.WriteProfileInt("Settings", "ApplyLimitToList", m_bApplyLimit);
    theApp.WriteProfileInt("Settings", "HideTrayIcon", m_bHideTrayIcon);
	theApp.WriteProfileInt("Settings", "CookieTimeout", m_nCookieTimeout);

	// Activity Log
	theApp.WriteProfileInt("Settings", "CreateLog", m_bCreateLog);
	theApp.WriteProfileInt("Settings", "LimitLog", m_bLimitLog);
	theApp.WriteProfileInt("Settings", "LogLevel", m_nLogLevel);
	theApp.WriteProfileInt("Settings", "LogSize", m_nLogSize);
#endif

#endif
