/*************************************************************************
*
* YPOPs!
*
* YPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	DataHolder.h
*
* Purpose : Retrieve/Save the configured settings
*
* Authors : Anuj Seth
* 			K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#ifndef _DATAHOLDER_H_
#define _DATAHOLDER_H_

#include "vsscanf.h"	// Added by ClassView
#include "Defines.h"
#include "inifile.h"

enum EMAIL_CATEGORY { EC_ALL = 'a', EC_UNREAD = 'u', EC_FLAGGED = 'f'};

// misc global defaults
#define DEF_SAWDISCLAIMER			FALSE
#define DEF_ALWAYSONTOP				FALSE
#define DEF_HIDETRAYICON			FALSE
#define DEF_QUIETMODE				TRUE
#define DEF_USERAGENT				"Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.0)"
#define DEF_YAHOODOMAIN             "yahoo.com"

// proxy defaults
#define DEF_HTTPPROXYPORT			80
#define DEF_HTTPSPROXYPORT			80
#define DEF_USEPROXY				FALSE
#define DEF_PROXYAUTH				FALSE

// network defaults
#define DEF_NETWORKIP				"127.0.0.1"
#define DEF_POP3PORT				110
#define DEF_SMTPPORT				25
#define DEF_ENABLESMTP				TRUE

// misc user defaults
#define DEF_MAXEMAILS				40
#define DEF_EMPTYTRASH				FALSE
#define DEF_EMPTYBULKMAIL			FALSE
#define DEF_BULKMAIL				TRUE
#define DEF_INBOX					TRUE
#define DEF_EMAILCATEGORY			EC_ALL
#define DEF_LEAVEUNREAD				FALSE
#define DEF_SAVEEMAILCOPY			FALSE
#define DEF_ENABLEUIDL				TRUE
#define DEF_IGNOREEXACTSIZE			TRUE
#define DEF_APPLYLIMITTOLIST		TRUE
#define DEF_COOKIETIMEOUT			12000

// activity log
#define DEF_CREATELOG				TRUE
#define DEF_LIMITLOG				FALSE
#define DEF_LOGLEVEL				1
#define DEF_LOGSIZE					50

extern CIniFile inf;


class CDataHolder  
{
public:
	CDataHolder();
	virtual ~CDataHolder();
	int  Load(const char *user_a = NULL);
	void Save(const char *user_a = NULL);
    void DumpToLog();

	// Persistent Data
	BOOL            m_bSawDisclaimer;

	CStringEx		m_strHttpProxyHost;
	UINT			m_nHttpProxyPort;
	CStringEx		m_strHttpsProxyHost;
	UINT			m_nHttpsProxyPort;
	BOOL			m_bUseProxy;
	CStringEx		m_UserAgent;
	UINT			m_nMaxEmails;
	BOOL			m_bEmptyTrash;
	BOOL			m_bEmptyBulkMail;
	BOOL			m_bBulkMail;
	BOOL			m_bInbox;
	EMAIL_CATEGORY	m_nEmailCategory;
	BOOL			m_bProxyAuth;
	CStringEx		m_strProxyAuthPassword;
	CStringEx		m_strProxyAuthUsername;
	BOOL			m_bQuietMode;
	CStringEx		m_strNetworkIPAddr;
	UINT			m_nNetworkPort;
	BOOL			m_bLeaveUnread;
	BOOL			m_bAutoStartWithWindows;
	UINT			m_nSmtpPort;
	CStringEx       m_strFolderList;
    BOOL            m_bAlwaysOnTop;
    BOOL            m_bEnableSmtp;
    BOOL            m_bSaveEmailCopy;
    BOOL            m_bEnableUIDL;
    BOOL            m_bHideTrayIcon;
    BOOL            m_bIgnoreSize;
    BOOL            m_bApplyLimit;
	CStringEx       m_sYahooDomain;

	UINT			m_nCookieTimeout;

	// Activity Log
	BOOL			m_bCreateLog;
	BOOL			m_bLimitLog;
	UINT			m_nLogLevel;
	UINT			m_nLogSize;

private:
    CStringEx stripQ(const char *strA);
    int loadOld();
    void LoadSection(const char *section);
	void SaveSection(const char *section);
	void ReadStr (CStringEx *svar, CCHR *pSection, CCHR *pKey, char *Default);
	void ReadString (CStringEx *svar, CCHR *pSection, CCHR *pKey, char *Default);
	void ReadBool(BOOL *var, CCHR *pSection, CCHR *pKey, int Default);
	void ReadInt (UINT *var, CCHR *pSection, CCHR *pKey, int Default);
    void WriteStr(CCHR *pSection, CCHR *pKey, CCHR *pValue, CCHR *pDefault);
	void WriteBool(CCHR *pSection, CCHR *pKey, UINT Value, UINT Default);
	void WriteInt(CCHR *pSection, CCHR *pKey, int Value, int Default);

	char			*inipath;

};

#endif

