/*************************************************************************
*
* YPOPs!
*
* YPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	ServiceClient.cpp
*
* Purpose : Class to service a new POP3 connection
*           - Thread-safe implementation
*
* Authors : Anuj Seth & K Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/

#include "stdafx.h"
#ifdef WIN32
#include <winsock.h>
#include "resource.h"
#include "ProgressWnd.h"
#include "mainfrm.h"
#else // #ifdef WIN32
#include <errno.h>
#include <string.h>
#endif

#include "ServiceClient.h"
#include "DataHolder.h"
#include "WebBrowser.h"
#include "LogFile.h"
#include "YahooSessionCache.h"

#ifndef WIN32
#include "YPOPs.h"
#endif
extern CDataHolder		 settings;
extern LogFile			 logFile;
extern YahooSessionCache sessionCache;

#ifdef WIN32
extern CMainFrame		 *gp_MainFrame;
#else
typedef void* (*THREAD_FUNCTION)(void *);
typedef void* THREAD_PARAMETER;
#define IDS_POP3_USER_OK         "+OK User name accepted, password please\r\n"
#define IDS_POP3_UNKNOWN_AUTH    "-ERR Unknown AUTHORIZATION state command\r\n"
#define IDS_LOGIN_UNKNOWN_ERROR	 "-ERR Unknown error"
#define IDS_LOGIN_FAILED		 "-ERR Login failed"
#define IDS_LOGIN_WRONG_PASSWORD "-ERR Incorrect Password"
#define IDS_LOGIN_WRONG_USERNAME "-ERR Invalid User Name"
#define IDS_INBOX_ERROR			 "-ERR Inbox Error"
#define IDS_USER_ABORTED		 "-ERR User aborted operation"
#endif

#ifdef YPOPS_USE_PURIFY
#include "pure.h"
#endif

/*******************************************************************
* Function  : ListenThread()
* Parameters: -
* Returns   : 0
* Purpose   : A thread that listens for POP3 client requests
*******************************************************************/
UINT ListenThread(void *lpParameter)
{
	struct sockaddr_in serv;
	struct sockaddr_in cli;
	int    listenfd = 0;
	int    sockfd   = 0;
	int    clilen   = 0;
	int    result   = 0;
	int    retVal   = 0;

#ifndef WIN32
	int howStarted = 0;
	( lpParameter != NULL ) ? ( howStarted =  *((int*)lpParameter) ) : ( howStarted = 0 );
#endif
	if((listenfd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
        AfxEndThread(0);
#else
		printf("Unable to create socket.Error reason = %s\n",strerror(errno));
		if ( howStarted == 1 ) 
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif
	}
	
	memset(&serv, 0, sizeof(serv));
	serv.sin_family 	 = AF_INET;
	if (settings.m_strNetworkIPAddr.Equals("ANY"))
		serv.sin_addr.s_addr = INADDR_ANY;
	else
		serv.sin_addr.s_addr = inet_addr(settings.m_strNetworkIPAddr.c_str());
//	serv.sin_addr.s_addr = INADDR_ANY; // this was the code for unix!
	serv.sin_port		 = htons(settings.m_nNetworkPort);
	
	if (bind(listenfd,(struct sockaddr *) &serv, sizeof(serv)) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
        AfxEndThread(0);
#else
		printf("Unable to bind to POP3 port %d, reason = %s\n",settings.m_nNetworkPort, strerror(errno));
		close(listenfd);
		if ( howStarted == 1 )
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif
	}

#ifndef WIN32
	/* we use this for shutdown purposes */
	listenThreadPort(&listenfd);
#endif

	
	if (listen(listenfd, 25) < 0)
	{
#ifdef WIN32
		::AfxMessageBox(IDS_POP3_SOCKET_ERROR, MB_OK | MB_ICONSTOP);
		closesocket(listenfd);
        AfxEndThread(0);
#else
		printf("Unable to listen.Error reason = %s\n",strerror(errno));
		close(listenfd);
		if ( howStarted == 1 ) 
		{
			pthread_exit((void*)NULL);
		}
		else
		{
			return(0);
		}
#endif

	}

	while(1)
	{
		clilen = sizeof(cli);

#ifdef WIN32
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli,&clilen)) < 0)
#else
		if((sockfd = accept(listenfd,(struct sockaddr *) &cli,(socklen_t*)&clilen)) < 0)
#endif
		{
			logFile.Write(LOG_BASIC,"POP3 accept() failed: %s\n", strerror(errno));
			break;
		}
		else
		{
			
			/* Edit SH. Dump the settings everytime if advanced mode on */
			if (settings.m_nLogLevel==LOG_ADVANCED)
				settings.DumpToLog();

#ifdef WIN32
			::AfxBeginThread(ServiceClientThread, (LPVOID)sockfd);
#else
			pthread_t threadId;
			pthread_create(&threadId,NULL,(THREAD_FUNCTION)ServiceClientThread,(THREAD_PARAMETER)sockfd);
#endif
		}
	}
			
	// Exiting
#ifdef WIN32
	::PostMessage(::AfxGetMainWnd()->GetSafeHwnd(), WM_QUIT, 0, 0);
    AfxEndThread(0);
    return 0;
#else
	close(listenfd);
	if ( howStarted == 1 )
	{
		pthread_exit((void*)NULL);
	}
	else
	{
		return(0);
	}
#endif
}

/*******************************************************************
* Function  : ProgressThread
* Parameters: lpParameter -> The data object containing all the
*             information this thread required
* Returns   : -
* Purpose   : To show the progress of the service in a thread. A
*             thread is being used to ensure that the dialog keeps
*             painting itself without any problems.
*******************************************************************/
#ifdef WIN32
UINT ProgressThread(void *lpParameter)
{
	PROGRESS_DATA *data = reinterpret_cast<PROGRESS_DATA*>(lpParameter);

	if(settings.m_bQuietMode)
	{
		data->progress = NULL;
	}
	else
	{
		data->progress = new CProgressWnd(::AfxGetMainWnd(), "", TRUE);
		data->progress->SetProgressWnd(data->progress);
		data->progress->CenterWindow();
	}
	
	data->init.SetEvent();

	while(::WaitForSingleObject(data->destroy, 0) != WAIT_OBJECT_0)
	{
		if(data->progress != NULL)
		{
			data->progress->PeekAndPump();

			if(data->progress->Cancelled())
			{
				break;
			}
		}

		Sleep(500);
	}

	delete data;
	data = NULL;

    AfxEndThread(0);
    return 0;
}

CStdString ErrorString(DWORD err)
{
     CString Error;
     LPTSTR s;
     if(::FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER |
			FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			err,
			0,
			(LPTSTR)&s,
			0,
			NULL) == 0)
	{
		 // Unknown error code %08x (%d)
		 Error.Format("Unknown O/S Error %08x (%d)", err, LOWORD(err));
	}
    else
	{
		 LPTSTR p = _tcschr(s, _T('\r'));
		 if(p != NULL)
		 { /* lose CRLF */
			p = _T('\0');
		 } /* lose CRLF */
		 Error = s;
		 ::LocalFree(s);
	}
    return Error;
} // ErrorString
#endif

const CStdString socket_error()
{
#ifdef WIN32
	return ErrorString(WSAGetLastError());
#else
	return CStdString(strerror(errno));
#endif
}

void ServiceClient::closeit()
{
	if (sock == (SOCKET)0)
		return; // already closed
	logFile.Write(LOG_ADVANCED, "<%s> closing POP3 socket %d", LPCTSTR(username), (int)sock);
#ifdef WIN32
	if (closesocket(sock) != 0)
#else
	if (close(sock) != 0)
#endif
	{
		logFile.Write(LOG_BASIC, "<%s> Could NOT close POP3 socket", LPCTSTR(username));
		logFile.Write(LOG_BASIC, "<%s> due to socket error: %s",
			LPCTSTR(username), LPCTSTR(socket_error()));
	}
	sock = (SOCKET)0;
}

void ServiceClient::sendit(SOCKET &sock, CStdString str, int len)
{
	logFile.Write(LOG_ADVANCED, "<%s> Sending POP3 response to socket %d: %s",
		LPCTSTR(username), (int)sock, str.Buffer());
	/* send response to the client */
	if (send(sock, LPCTSTR(str), len, 0) < 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Could NOT send this POP3 response: %s",
			LPCTSTR(username), str.Buffer());
		logFile.Write(LOG_BASIC, "<%s> due to socket error: %s",
			LPCTSTR(username), LPCTSTR(socket_error()));
	}
	else
	{
		logFile.Write(LOG_ADVANCED, "<%s> POP3 response sent: %s",
			LPCTSTR(username), str.Buffer());
	}
}

void ServiceClient::sendit(SOCKET &sock, CStdString str)
{
	sendit(sock, str, str.GetLength());
}

void ServiceClient::SendResponse()
{
	/* send response to the client */
	sendit(sock, str);
}

void ServiceClient::SendResponse(const char *cstr)
{
	str = cstr;
	SendResponse();
}

#ifdef WIN32
void ServiceClient::SendResponse(int errorCode)
{
	str.LoadString(errorCode);
	SendResponse();
}
#endif

/*******************************************************************
* Function  : ServiceClientThread
* Parameters: lpParameter -> The client socket
* Returns   : Return value from service.Execute()
* Purpose   : To service a POP3 clients request. This is a thread
*             which is spawned by ListenThread()
*******************************************************************/
UINT ServiceClientThread(void *lpParameter)
{
	int retVal;
	int sock =0;
	try
	{
		SOCKET sock = (reinterpret_cast<SOCKET>(lpParameter));
		ServiceClient service(sock);
		retVal = service.Execute();
	}
	catch(std::bad_alloc& e1)
	{
		logFile.Write(LOG_BASIC, "Caught std::bad_alloc Exception %s - aborting POP3 request",
			LPCTSTR(e1.what()));
		retVal = 1;
	}
#ifdef WIN32
	catch(CMemoryException e2)
	{
		char buf[255];
		e2.GetErrorMessage(buf, 253);
		buf[254] = '\0';
		logFile.Write(LOG_BASIC, "Caught CMemoryException Exception %s - aborting POP3 request",
			LPCTSTR(buf));
		retVal = 1;
	}
	catch(CException e3)
	{
		char buf[255];
		e3.GetErrorMessage(buf, 253);
		buf[254] = '\0';
		logFile.Write(LOG_BASIC, "Caught CException Exception %s - aborting POP3 request",
			LPCTSTR(buf));
		retVal = 1;
	}
#endif
	catch(exception& e4)
	{
		logFile.Write(LOG_BASIC, "Caught exception Exception %s - aborting POP3 request",
			LPCTSTR(e4.what()));
		retVal = 1;
	}
	catch(...)
	{
		/* 
		 * This catch will catch only C++ exceptions.
		 * I don't expect it to catch any C runtime
		 * library exceptions which could be caused
		 * due to the bugs in my code :(
		 */
		logFile.Write(LOG_BASIC, "Caught Program Exception - aborting POP3 request");
		retVal = 1;
	}
#ifdef WIN32
	closesocket(sock);
	AfxEndThread(retVal);
#else
	close(sock);
	sleep(1);
	pthread_exit(NULL);	
#endif
    return 0;
}


/*******************************************************************
* Function  : ServiceClient() constructor
* Parameters: sock -> POP3 Client socket
* Returns   : -
* Purpose   : Initialize the ServiceClient object and send the
*             "WELCOME" greeting to the POP3 client.
*******************************************************************/
ServiceClient::ServiceClient(SOCKET sock)
{
	*username = '\0';
	this->sock = sock;
#ifdef WIN32
	str.LoadString(IDS_POP3_READY);
#else
	str = "+OK POP3 YPOPs! proxy ready\r\n";
#endif
	sendit(this->sock, str);
}

/*******************************************************************
* Function  : ServiceClient() destructor
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
ServiceClient::~ServiceClient()
{
	closeit();
}

/*******************************************************************
* Function  : ReadUser()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : To read the "USER <user>" data from the socket
*******************************************************************/
int ServiceClient::ReadUser()
{
	char *ptr;
	memset(buf,'\0', 1023);
	buflen = recv(sock, buf, 1023, 0);

	if(buflen <= 0)
	{
		return 1;
	}

	if( (ptr = strchr(buf, ' ')) != NULL && strnicmp(buf, "USER", 4) == 0)
	{
		/* Fix provided by glibdud@users.sourceforge.net */
		sscanf(buf, "%*4s %s\r\n", username);
		SendResponse(IDS_POP3_USER_OK);
	}
	else
	{
		logFile.Write(LOG_BASIC, "Unknown AUTHORIZATION state command: %s", buf);
		SendResponse(IDS_POP3_UNKNOWN_AUTH);
		return 2;
	}

	return 0;
}

/*******************************************************************
* Function  : ReadPass()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : To read the "PASS <password>" data from the socket
*******************************************************************/
int ServiceClient::ReadPass()
{
	char *ptr;
	memset(buf,'\0', 1023);
	buflen = recv(sock, buf, 1023, 0);

	if( (ptr = strchr(buf, ' ')) != NULL && strnicmp(buf, "PASS", 4) == 0)
	{
		/*The original sscanf is replaced to fix
		  the 'space in password problem' (by ltd)*/
		ptr=strstr(buf,"\r\n");
		*ptr=0;
		ptr=strchr(buf,' ');
		while (*(++ptr)==' ');
		strcpy(password,ptr);
	}
	else
	{
		logFile.Write(LOG_BASIC, "Unknown AUTHORIZATION state command: %s", buf);
		SendError(IDS_POP3_UNKNOWN_AUTH);
		return 1;
	}

	return 0;
}

/*******************************************************************
* Function  : Execute()
* Parameters: -
* Returns   : 0 - Success
*             1 - Error
* Purpose   : The main function which processes the POP3 clients
*             requests.
*******************************************************************/
int ServiceClient::Execute()
{
	int i;
#ifdef WIN32
	CString tooltip;
#endif

	do
	{
		i = ReadUser();

		if(i == 1)
		{
			return 1;
		}

	}while(i != 0);

	logFile.Write(LOG_BASIC, "Starting POP3 session for \"%s\".", username);
	
#ifdef WIN32
	tooltip.Format("[%s] Starting POP3 session", username);

	PROGRESS_DATA *data = new PROGRESS_DATA;

	::AfxBeginThread(ProgressThread, data);
	::WaitForSingleObject(data->init, INFINITE);

	CString caption;
	caption.Format("Email download status for %s", username);

	if(data->progress != NULL)
	{
		data->progress->SetCaption(caption);
		data->progress->SetRange(0, 100);
		data->progress->SetText("Waiting for password...");
	}

	if(ReadPass() != 0)
	{
		data->destroy.SetEvent();
		return 1;
	}

	logFile.Write(LOG_ADVANCED, "<%s> Password received from client.", username);

	if(data->progress != NULL)
	{
		data->progress->SetText("Connecting to Yahoo! Mail...");
	}
#else
	if(ReadPass() != 0)
	{
	    logFile.Write(LOG_ADVANCED,"Unable to read password\n");
		return 1;
	}

	logFile.Write(LOG_ADVANCED, "<%s> Password received from client.\n", username);
#endif

    settings.Load(username);

	int retVal = browser.ConnectToYahoo(username, password);

#ifdef WIN32
	if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
	{
		/* User clicked on the Cancel button */
		closeit();

		if(data != NULL)
			data->destroy.SetEvent();

		return 0;
	}
#endif

	switch(retVal)
	{
	case WEB_LOGIN_OK:
		/* Perfect! Now we proceed with the actual work */
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail succeeded", username);
#endif
		break;

	case WEB_LOGIN_FAILED:
		closeit();
#ifdef WIN32
		tooltip.Format("[%s] Could not connect to Yahoo!", username);
		data->destroy.SetEvent();
#endif
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed.", username);
        browser.DeleteSession();
		return 1;
		
	case WEB_UNKNOWN_ERROR:
		closeit();
#ifdef WIN32
		tooltip.Format("[%s] Could not connect to Yahoo!. Unknown Error (Raise bug report and attach ypops.log)", username);
		data->destroy.SetEvent();
#endif
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed (Unknown error).", username);
        browser.DeleteSession();
		return 1;

	case WEB_WRONG_PASSWORD:
		SendError(IDS_LOGIN_WRONG_PASSWORD);
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to an incorrect password.", username);
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail failed due to an incorrect password", username);
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;

	case WEB_WRONG_USERNAME:
		SendError(IDS_LOGIN_WRONG_USERNAME);
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to an incorrect logid id.", username);
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail failed due to an incorrect logid id", username);
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;

	default: /* Treat as unknown error! */
		closeit();
		logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail failed due to a bug in YPOPs!.", username);
#ifdef WIN32
		tooltip.Format("[%s] Login to Yahoo Mail failed due to a bug in YPOPs! Raise bug report and attach ypops.log", username);
		data->destroy.SetEvent();
#endif
        browser.DeleteSession();
		return 1;
	}

#ifdef WIN32
	gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

	//EDIT: Shane .h : Save the session even if we aren't using SMTP
    //if(settings.m_bEnableSmtp)
    //{
	//Session is already saved in connectToYahoo (if it was successful)
	//browser.SaveSession();
    //}

#ifdef WIN32
	if(data->progress != NULL)
		data->progress->SetText("Analyzing the Inbox folder for emails...");
#endif

	if(settings.m_bInbox)
	{
		logFile.Write(LOG_BASIC, "<%s> Analyzing the INBOX folder", username);

#ifdef WIN32
		tooltip.Format("[%s] Analyzing the INBOX folder", username);
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

        retVal = browser.FetchMailboxList(yahooMbox[MB_INBOX]);
		
		if (retVal == WEB_LOGIN_OK || retVal == WEB_INBOX_EMPTY)
			logFile.Write(LOG_BASIC, "<%s> INBOX folder analyzed", username);

#ifdef WIN32	
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);

			/* User clicked on the Cancel button */
			closeit();

			if(data != NULL)
				data->destroy.SetEvent();

			gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
			return 0;
		}
#endif

		switch(retVal)
		{
		case WEB_INBOX_FETCH_OK:
			/* You've got Mail! */
			break;
			
		case WEB_INBOX_EMPTY:
			/* You've got no mail :( */	
			break;
			
		case WEB_INBOX_FETCH_FAILED:
		case WEB_UNKNOWN_ERROR:
        default:

			SendError(IDS_INBOX_ERROR);
#ifdef WIN32
			gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
			if(data != NULL)
				data->destroy.SetEvent();
#endif
			return 1;
		}
	}

	if(settings.m_bBulkMail)
	{
#ifdef WIN32
		if(data->progress != NULL)
			data->progress->SetText("Analyzing the Bulk Mail folder for emails...");

        tooltip.Format("[%s] Analyzing the BulkMail folder", username);
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));

#endif

		logFile.Write(LOG_BASIC, "<%s> Analyzing the BulkMail folder", username);
		retVal = browser.FetchMailboxList(yahooMbox[MB_BULKMAIL]);
		logFile.Write(LOG_BASIC, "<%s> BulkMail folder analyzed", username);
	}

#ifdef WIN32	
	if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
	{
		logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);
	
		/* User clicked on the Cancel button */
		closeit();

		if(data != NULL)
			data->destroy.SetEvent();

		gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
		return 0;
	}
#endif

    /* Analyze all the user defined folders */
	int prevLoc = 0;

	while(1)
	{
		CStdString folderName;

		int i = settings.m_strFolderList.Find("|", prevLoc);

		if(i == -1)
		{
			folderName = settings.m_strFolderList.Mid(prevLoc);

			if(folderName.IsEmpty())
				break;
		}
		else
		{
			folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
		}

#ifdef WIN32		
		if(data->progress != NULL)
		{
			str.Format("Analyzing the %s folder for emails...", LPCTSTR(folderName));
			data->progress->SetText(str);
		}
#endif

		logFile.Write(LOG_BASIC, "<%s> Analyzing the %s folder", username, LPCTSTR(folderName));

#ifdef WIN32
		tooltip.Format("[%s] Analyzing the %s folder", username, LPCTSTR(folderName));
		gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

        retVal = browser.FetchMailboxList(folderName);

		logFile.Write(LOG_BASIC, "<%s> %s folder analyzed", username, LPCTSTR(folderName));

#ifdef WIN32
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);
		
			/* User clicked on the Cancel button */
			closeit();

			if(data != NULL)
				data->destroy.SetEvent();

			gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
			return 0;
		}
#endif

		prevLoc = i + 1;

		if(i == -1)
		{
			break;
		}
	}

	/* Optional download of all emails may be turned off
	   if client doesn't care about exact message size 
	   (experimental feature by ltd) */
	if (!settings.m_bIgnoreSize) {
		/* 
		 * Download all emails because most of the POP3 commands
		 * that will be received from any POP3 client would
		 * require knowledge about the email size, etc. So it
		 * makes sense to download them.
		 */

		logFile.Write(LOG_BASIC, "<%s> Found %d emails", username, browser.GetMsgCount());

#ifdef WIN32
		if(data->progress != NULL)
			data->progress->SetRange(0, browser.GetMsgCount());
#endif

		if(browser.GetMsgCount() != 0)
		{
#ifdef WIN32
			tooltip.Format("[%s] Downloading emails", username);
			gp_MainFrame->m_TrayIcon.SetTooltipText(tooltip);
			retVal = browser.DownloadAllEmails(data->progress);
#else
			retVal = browser.DownloadAllEmails();
#endif
		}

#ifdef WIN32
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);

			/* User clicked on the Cancel button */
			closeit();

			if(data != NULL)
				data->destroy.SetEvent();

			gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
			return 0;
		}

		if(data->progress != NULL)
			data->progress->SetText("Servicing your mail clients requests...");
#endif
	}

	CStdString progressStr;

#ifdef WIN32
	tooltip.Format("[%s] Servicing email application requests", username);
	gp_MainFrame->m_TrayIcon.SetTooltipText(LPCTSTR(tooltip));
#endif

// Declare the variables needed to dump memory leak variables
#if defined(_DEBUG) && defined(WIN32)
    CMemoryState oldMemState, newMemState, diffMemState;
    oldMemState.Checkpoint();
#endif
#ifdef YPOPS_USE_PURIFY_SMTP
	PurifyNewLeaks();
	PurifyHeapValidate((unsigned int)PURIFY_HEAP_ALL, PURIFY_HEAP_BLOCKS_ALL, NULL);
	PurifyAllLeaks();
	PurifyAllInuse();
#endif

	SendResponse("+OK Mailbox open and analyzed\r\n");

	while(TRUE)
	{
		memset(buf, '\0', 1023);
		buflen = recv(sock, buf, 1023, 0);

		if(buflen <= 0)
		{
			/* Socket has been closed */
			break;
		}

		logFile.Write(LOG_ADVANCED, "<%s> Command received from email client: %s", username, buf);

#ifdef WIN32		
		if(data == NULL || (data->progress != NULL && data->progress->Cancelled()))
		{
			logFile.Write(LOG_ADVANCED, "<%s> Cancel event received", username);

			/* User clicked on the Cancel button */
			closeit();

			if(data != NULL)
				data->destroy.SetEvent();
	
			gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
			return 0;
		}

		if(data->progress != NULL)
		{
			progressStr.Format("Request: %s", buf);
			data->progress->SetText(progressStr);
		}
#endif
		if(strnicmp(buf, "STAT", 4) == 0)
		{
			retVal = ServiceSTAT();
		}
		else if(strnicmp(buf, "LIST", 4) == 0)
		{
			retVal = ServiceLIST(buf);
		}
		else if(strnicmp(buf, "RETR", 4) == 0)
		{
			retVal = ServiceRETR(buf);
		}
		else if(strnicmp(buf, "DELE", 4) == 0)
		{
			retVal = ServiceDELE(buf);
		}
		else if(strnicmp(buf, "NOOP", 4) == 0)
		{
			retVal = ServiceNOOP();
		}
		else if(strnicmp(buf, "RSET", 4) == 0)
		{
			retVal = ServiceRSET();
		}
		else if(strnicmp(buf, "QUIT", 4) == 0)
		{
#ifdef WIN32
			if(data->progress != NULL && browser.AnyEmailsToDelete())
			{
				progressStr.Format("Deleting emails from Yahoo...");
				data->progress->SetText(progressStr);
			}
#endif
			retVal = ServiceQUIT();

			/* Mark unread */
			if(settings.m_bLeaveUnread)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Marking emails as unread on server...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.MarkUnread(yahooMbox[MB_INBOX]);
				browser.MarkUnread(yahooMbox[MB_BULKMAIL]);

				prevLoc = 0;

				while(1)
				{
					CStdString folderName;
					int i = settings.m_strFolderList.Find("|", prevLoc);

					if(i == -1)
					{
						folderName = settings.m_strFolderList.Mid(prevLoc);

						if(folderName.IsEmpty())
						{
							break;
						}
					}
					else
					{
						folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
					}

					prevLoc = i + 1;

					browser.MarkUnread(folderName);
					
					if(i == -1)
					{
						break;
					}
				}
			}

			/* Empty Trash */
			if(settings.m_bEmptyTrash)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Emptying trash...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.EmptyTrash();
			}

			if(settings.m_bEmptyBulkMail)
			{
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Emptying bulk mail...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.EmptyBulkMail();
			}

			/* Sign out */
			if(settings.m_nCookieTimeout==0) {
#ifdef WIN32
				if(data->progress != NULL)
				{
					progressStr.Format("Logging off...");
					data->progress->SetText(progressStr);
				}
#endif
				browser.LogOff();
			}

			break;
		}
		else if(strnicmp(buf, "TOP", 3) == 0)
		{
			retVal = ServiceTOP(buf);
		}
		else if(strnicmp(buf, "UIDL", 4) == 0)
		{
			retVal = ServiceUIDL();
		}
		else
		{
			/* Catch all */
			SendResponse("-ERR Unknown command\r\n");
		}
	}

	/* The following call to save session might seem redudant,
	 * but it actually updates the last access time for the 
	 * session.
	 */
    
	//EDIT: Shane. H - We don't want to update the last access time anymore
	//if(settings.m_bEnableSmtp)
    //{
	//browser.SaveSession();
	//}

	closeit();
#ifdef WIN32
	gp_MainFrame->m_TrayIcon.SetTooltipText("YPOPs!");
	data->destroy.SetEvent();
#endif

#if defined(_DEBUG) && defined(WIN32)
    newMemState.Checkpoint();
    if( diffMemState.Difference( oldMemState, newMemState ) )
    {
        TRACE( "Memory leaked!\n" );
		diffMemState.DumpAllObjectsSince();
    }
#endif
#ifdef YPOPS_USE_PURIFY
	PurifyNewLeaks();
	PurifyHeapValidate((unsigned int)PURIFY_HEAP_ALL, PURIFY_HEAP_BLOCKS_ALL, NULL);
#endif

    return 0;
}

/*******************************************************************
* Function  : SendError()
* Parameters: errorCode - The error message to send to the client
* Returns   : -
* Purpose   : To send an error message to the POP3 client
*******************************************************************/
void ServiceClient::SendError(const char *errstr)
{
	SendResponse(errstr);
	/* Close the socket as we cannot proceed any further */
	closeit();
}

#ifdef WIN32
void ServiceClient::SendError(int errorCode)
{
	str.LoadString(errorCode);
	SendError(str.c_str());
}
#endif

/*******************************************************************
* Function  : ServiceSTAT()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the STAT command
*******************************************************************/
int ServiceClient::ServiceSTAT()
{
	int totalMessages = browser.GetMsgCount();
	int undeletedMsgs = 0;
	int msgSize       = 0;
	Email *email      = NULL;

	for(int i = 0; i < totalMessages; i++)
	{
		email = browser.GetEmail(i + 1);

		if(email != NULL)
		{
			if(!email->IsDeleted())
			{
				undeletedMsgs++;
				msgSize += email->GetSize();
			}
		}
	}

	str.Format("+OK %d %d\r\n", undeletedMsgs, msgSize);
	SendResponse();

	return 0;
}

/*******************************************************************
* Function  : ServiceLIST()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the LIST command
*******************************************************************/
int ServiceClient::ServiceLIST(char *buf)
{
	int msgNumber = -1;
	Email *email  = NULL;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	if(msgNumber == -1)
	{
		/* Return entire list */
		SendResponse("+OK scan listing follows\r\n");

		int totalMessages = browser.GetMsgCount();
		int undeletedMsgs = 0;

		for(int i = 0; i < totalMessages; i++)
		{
			email = browser.GetEmail(i + 1);

			if(email != NULL)
			{
				if(!email->IsDeleted())
				{
					undeletedMsgs++;
					str.Format("%d %d\r\n", undeletedMsgs, email->GetSize());
					SendResponse();
				}
			}
		}

		SendResponse(".\r\n");
	}
	else
	{
		/* Return details for specified message only */
		email = browser.GetEmail(msgNumber);

		if(email != NULL && !email->IsDeleted() && (email->IsDownloaded() || settings.m_bIgnoreSize))
		{
			str.Format("+OK %d %d\r\n", msgNumber, email->GetSize());
			SendResponse();
		}
		else
		{
			SendResponse("-ERR no such message OR message marked as deleted OR message not downloaded\r\n");
		}
	}

	return 0;
}

/*******************************************************************
* Function  : ServiceRETR()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the RETR command
*******************************************************************/
int ServiceClient::ServiceRETR(char *buf)
{
	int msgNumber = -1;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	Email *email = browser.GetEmail(msgNumber);

	if(email == NULL)
	{
		SendResponse("-ERR no such message\r\n");
	}
	else if(email->IsDeleted())
	{
		str.Format("-ERR message %d has been marked as deleted\r\n", msgNumber);
		SendResponse();
	}
	else
	{
		/* RETR command requires the message to be downloaded */
		if (!email->IsDownloaded() && settings.m_bIgnoreSize)
			browser.DownloadEmail(msgNumber);

 		if(email->IsDownloaded())
		{
			if (email->SendMessage(sock))
				logFile.Write(LOG_ADVANCED,
					"<%s> Response sent: [Email (body length=%d) sent to client]",
					username, email->GetBodySize());
		}
		else
		{
			str.Format("-ERR message %d could not be downloaded\r\n", msgNumber);
			SendResponse();
		}
	}
	
	return 0;
}

/*******************************************************************
* Function  : ServiceDELE()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the DELE command
*******************************************************************/
int ServiceClient::ServiceDELE(char *buf)
{
	int msgNumber = -1;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);
	MESSAGE *msg = browser.GetMessage(msgNumber);

	if(msg == NULL)
	{
		str = "-ERR No such message\r\n";
	}
	else
	{
		if(msg->email.IsDeleted())
		{
			str.Format("-ERR message %d already deleted\r\n", msgNumber);
		}
		else
		{
			msg->email.SetDeleted(TRUE);
			str.Format("+OK message %d deleted\r\n", msgNumber);
		}
	}

	SendResponse();

	return 0;
}

/*******************************************************************
* Function  : ServiceNOOP()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the NOOP command
*******************************************************************/
int ServiceClient::ServiceNOOP()
{
	SendResponse("+OK\r\n");

	return 0;
}

/*******************************************************************
* Function  : ServiceRSET()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the RSET command
*******************************************************************/
int ServiceClient::ServiceRSET()
{
	Email *email;
	int msgCount = browser.GetMsgCount();

	for(int i = 0; i < msgCount; i++)
	{
		email = browser.GetEmail(i + 1);

		if(email != NULL)
		{
			email->SetDeleted(FALSE);
		}
	}

	str.Format("+OK YPOPs! now has %d messages for you\r\n", msgCount);
	SendResponse();

	return 0;
}

/*******************************************************************
* Function  : ServiceQUIT()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the QUIT command
*******************************************************************/
int ServiceClient::ServiceQUIT()
{
	int retVal=0;
    if(browser.AnyEmailsToDelete())
    {
		logFile.Write(LOG_ADVANCED, "<%s> Deleting marked emails", username);
		
		retVal+=browser.DeleteEmails(yahooMbox[MB_INBOX]);
		retVal+=browser.DeleteEmails(yahooMbox[MB_BULKMAIL]);
		
		int prevLoc = 0;
		
		while(1)
		{
			CStdString folderName;
			
			int i = settings.m_strFolderList.Find("|", prevLoc);
			
			if(i == -1)
			{
				folderName = settings.m_strFolderList.Mid(prevLoc);
				
				if(folderName.IsEmpty())
				{
					break;
				}
			}
			else
			{
				folderName = settings.m_strFolderList.Mid(prevLoc, i - prevLoc);
			}
			
			/* retVal will stay 0 if all email were deleted successfully */
			retVal+=browser.DeleteEmails(folderName);
			
			prevLoc = i + 1;
			
			if(i == -1)
			{
				break;
			}
		}
	}

	if (retVal)
		str = "-ERR YPOPs! signing off. Some email not deleted.\r\n";
	else
		str = "+OK YPOPs! signing off. Hope you enjoyed this program.\r\n";

	SendResponse();

	/* 
	 * The thread will automatically be closed; because we
	 * have closed the socket. The recv() call will fail
	 * resulting in the thread to be closed.
	 */

	return 0;
}

/*******************************************************************
* Function  : ServiceTOP()
* Parameters: buf - The request line from the client
* Returns   : 0
* Purpose   : To send the client the response to the TOP command
*******************************************************************/
int ServiceClient::ServiceTOP(char *buf)
{
	int msgNumber = -1;
	int lines	  = -1;

	sscanf(buf, "%*3s %d %d\r\n", &msgNumber, &lines);

	Email *email = browser.GetEmail(msgNumber);

	if(email == NULL)
	{
		SendResponse("-ERR no such message\r\n");
	}
	else if(lines < 0)
	{
		SendResponse("-ERR invalid number of lines\r\n");
	}
	else if(email->IsDeleted())
	{
		str.Format("-ERR message %d has been marked as deleted\r\n", msgNumber);
		SendResponse();
	}
	else
	{
		/* TOP command require the message to be downloaded 
		   or at least headers if requested number of lines is 0 */
		if (lines) {
			if (!email->IsDownloaded() && settings.m_bIgnoreSize)
				browser.DownloadEmail(msgNumber);
			if (!email->IsDownloaded())
			{
					str.Format("-ERR message %d could not be downloaded\r\n", msgNumber);
					SendResponse();
					return 0;
			}
		} else {
			if (!email->IsHeaderDownloaded() && settings.m_bIgnoreSize)
				browser.DownloadEmail(msgNumber, TRUE);
			if (!email->IsHeaderDownloaded())
			{
				str.Format("-ERR headers of message %d could not be downloaded\r\n", msgNumber);
				SendResponse();
					return 0;
			}
		}
	
		if (email->SendMessage(sock, lines))
			logFile.Write(LOG_ADVANCED, "<%s> Response sent: [%d lines of requested email]", username, lines);
	}
	
	return 0;
}

/*******************************************************************
* Function  : ServiceUIDL()
* Parameters: -
* Returns   : 0
* Purpose   : To send the client the response to the UIDL command
*******************************************************************/
int ServiceClient::ServiceUIDL()
{
    if(!settings.m_bEnableUIDL)
    {
		SendResponse("-ERR UIDL support is currently disabled in YPOPs");
        return 0;
    }

	int msgNumber = -1;
	Email *email  = NULL;

	sscanf(buf, "%*4s %d\r\n", &msgNumber);

	if(msgNumber == -1)
	{
		/* Return entire list */
		SendResponse("+OK scan listing follows\r\n");

		int totalMessages = browser.GetMsgCount();
		int undeletedMsgs = 0;

		for(int i = 0; i < totalMessages; i++)
		{
			email = browser.GetEmail(i + 1);

			if(email != NULL)
			{
				if(!email->IsDeleted())
				{
					undeletedMsgs++;
					str.Format("%d %s\r\n", undeletedMsgs, email->GetMsgId());
					SendResponse();
				}
			}
		}

		SendResponse(".\r\n");
	}
	else
	{
		/* Return details for specified message only */
		email = browser.GetEmail(msgNumber);

		if(email != NULL && !email->IsDeleted())
		{
			str.Format("+OK %d %s\r\n", msgNumber, email->GetMsgId());
			SendResponse();
		}
		else
		{
			SendResponse("-ERR no such message OR message marked as deleted OR message not downloaded\r\n");
		}
	}

	return 0;
}
