/*************************************************************************
*
* YPOPs!
*
* YPOPs! is an application that provides POP3 access to Yahoo! Mail.
* You do not need to enable POP3 access with Yahoo! to use this program.
* This program works within the license agreement of Yahoo! to provide
* you with POP3 access.
*
* YPOPs! emulates a POP3 mail server and should work with most of the
* email clients. Since YPOPs! is not a full-fledged POP3 mail server
* you might find it not working with all mail clients. Do keep us informed
* about such mail clients so that we can enhance YPOPs!
*
* Official Website: http://yahoopops.sourceforge.net
*
*-------------------------------------------------------------------------
*
* Name    :	WebBrowser.cpp
*
* Purpose : A simple class specifically designed to connect to
*           Yahoo! Mail and download/submit emails.
*
* Authors : Anuj Seth, Daniel Bryg, K.Shyam
*
* License : GNU Public License (Version 2 or later)
*
*-------------------------------------------------------------------------
*
* THIS SOFTWARE IS PROVIDED ``AS IS'' BY THE AUTHORS AND INDEPENDENT
* CONTRIBUTORS AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
* LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A 
* PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR OR 
* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
* EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
* PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
* PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
* LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
* NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
* SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*
*************************************************************************/
#include "stdafx.h"
#ifdef WIN32
#include <direct.h> // for getcwd()
#else
#include <unistd.h> // for getcwd()
#include <fcntl.h> // for open()
#include <cerrno> // for errno
#include <cstdio> // for perror()
#endif
#include <sys/stat.h> // for _stat()
#include <openssl/md5.h>
#include "YPOPs.h"
#include "HttpClient.h"
#include "YahooStuff.h"
#include "DataHolder.h"
#include "LogFile.h"
#include "YahooSessionCache.h"
#include "SendVerify.h"
#include "YahooErrorDlg.h"
#include "WebBrowser.h"

extern CDataHolder settings;
extern LogFile     logFile;
extern YahooSessionCache sessionCache;

#ifdef WIN32
CCriticalSection CWebBrowser::csConnect;
#endif

#ifdef WIN32
#define sleep(s) Sleep(s*1000)
#endif

/*******************************************************************
* Function  : CWebBrowser::CWebBrowser
* Parameters: -
* Returns   : -
* Purpose   : Initialize the web browser object
*******************************************************************/
CWebBrowser::CWebBrowser():	
    msgHead(NULL),
    msgTail(NULL),
    msgCount(0),
	downloadedCount(0)
{
	/* Dont create a new session everytime - Shane H.
	session = new HttpSession; */
}

/*******************************************************************
* Function  : CWebBrowser::~CWebBrowser()
* Parameters: -
* Returns   : -
* Purpose   : Object cleanup
*******************************************************************/
CWebBrowser::~CWebBrowser()
{
	MESSAGE *ptr  = msgHead;
	MESSAGE *next = NULL;
	
	while(ptr != NULL)
	{
		next = ptr->next;
		delete ptr;
		ptr = next;
	}

	msgHead = NULL;
	msgTail = NULL;
}

CStringEx CWebBrowser::MaskPwd(CStringEx data)
{
	CStringEx ret = data;
	CStringEx lbl = "passwd=";
	int i1 = ret.Find(lbl);
	int i2 = ret.Find("&", i1 + 1);
	if (i1 < 0)
		return ret;
	i1 += lbl.GetLength();
	if (i2 < 0)
		i2 = ret.GetLength();
	for(int i=i1; i < i2; i++)
		ret.SetAt(i, 'X');
	return ret;
}

/*******************************************************************
* Function  : Lock/Unlock
* Parameters: none
* Returns   : void
* Purpose   : To lock/unlock critical sections to prevent multiple
*             sessions from clobbering one another (Avoid contention
*             in connection between POP and SMTP threads).
*******************************************************************/
void CWebBrowser::Lock()
{
#ifdef WIN32
	csConnect.Lock();
#else
	int f;
	while(TRUE)
	{
		if ((f = open("./ypops.lock", O_WRONLY | O_CREAT | O_EXCL)) < 0)
		{
			if (errno != EEXIST)
			{
				logFile.Write(LOG_BASIC, "<%s> locking failed: %s", LPCTSTR(username),
                    strerror(errno));
				logFile.Write(LOG_BASIC, "exiting due to lock falure");
				exit(1);
			}
			sleep(1);
		}
		else
		{
			close(f); // no need to hold it open
			break; // got lock
		}
	}
#endif
}

void CWebBrowser::Unlock()
{
#ifdef WIN32
		csConnect.Unlock();
#else
		remove("./ypops.lock"); // use unlink() ?
#endif
}

/*******************************************************************
* Function  : ConnectToYahoo
* Parameters: user     -> Username sent by the POP3 client
*             password -> Password sent by the POP3 client
* Returns   : WEB_LOGIN_FAILED   -> Unable to login to Yahoo
*             WEB_UNKNOWN_ERROR  -> Possible bug in YPOPs! caused
*                                   by a redesign in the Yahoo site.
*		      WEB_WRONG_PASSWORD -> Incorrect password
*		      WEB_WRONG_USERNAME -> Incorrect login id
*             WEB_LOGIN_OK       -> Login successful
* Purpose   : To login to Yahoo's website.
*******************************************************************/
int CWebBrowser::ConnectToYahoo(char *user, char *pass)
{

	CStringEx data;
	char *ptr = NULL;
	int	 retVal;

	username = user;
	password = pass;

	// Must be sure to unlock section before returning from function
	Lock();

	/* EDIT: Shane .H : Attempt to obtain existing session
	Must wait until the other thread has finished connecting and saved the session 
	NOTE: This is not efficient if more than one username is connecting at once */
	YAHOO_SESSION *ySession = sessionCache.GetSession(username, password);
	if(ySession != NULL)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Previous session recycled", LPCTSTR(username));
		SetSession(ySession);
		Unlock();
		return WEB_LOGIN_OK;
	}
	else
	{
		/* Create a new session if no existing session exists */
		logFile.Write(LOG_ADVANCED, "<%s> Creating a new session", LPCTSTR(username));
		session = new HttpSession;
	}

	/* Save cookies for future requests */
	session->SetSessionMgmt(TRUE);
   
	HttpClient c(session);
	HttpClientResponse resp;
	
	logFile.Write(LOG_BASIC, "<%s> Connecting to Yahoo...", LPCTSTR(username));
	
	/* 
	* Secure Password Transmission is enabled.
	* This is a slower operation but more secure.
	* The reason being that we first need to fetch
	* the http://mail.yahoo.com/ page and then 
	* login. We need to extract the ".challenge"
	* text from the page. After that we do the 
	* actual login. In the other process (non-MD5)
	* we do a login directly as there is no 
	* challenge text involved.
	*/
    CStringEx mailhost;
    mailhost.Format("mail.%s", LPCTSTR(settings.m_sYahooDomain));
    CStringEx loginhost;
    loginhost.Format("login.%s", LPCTSTR(settings.m_sYahooDomain));
    CStringEx mailurl;
    mailurl.Format("http://%s/?.intl=us", LPCTSTR(mailhost));

	if(c.DoGet(mailurl, resp) != 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
		Unlock();
		return WEB_LOGIN_FAILED;
	}
	
	DumpPage("prelogin",resp.GetBodyPtr());

	/*
	<form method=post action="https://login.yahoo.com/config/login?8dr403ev8v1hi" autocomplete=off name=login_form onsubmit="return hash(this,'http://us.rd.yahoo.com/reg/login0/no_suli/login/us/ym/*http://login.yahoo.com/config/login')">
	*/
	CStringEx loginUrl;
	ptr    = NULL;
	retVal = loginUrl.SearchRegEx(resp.GetBodyPtr(), 
		"form method=post action=\"", 
		&ptr,
		"form method=post action=\"%s\"");
	
	if(retVal != 0)
	{
		ptr    = NULL;
		retVal = loginUrl.SearchRegEx(resp.GetBodyPtr(), 
			"form method=\"post\" action=\"", 
			&ptr,
			"form method=\"post\" action=\"%s\"");
	}

	if(retVal != 0)
	{
		/* Something's wrong! */
		logFile.Write(LOG_BASIC, "<%s> Login failed (cannot find form URL!)", LPCTSTR(username));
		Unlock();
		return WEB_LOGIN_FAILED;
	}
		
	CStringEx u;
	ptr    = NULL;
	retVal = u.SearchRegEx(resp.GetBodyPtr(), 
		"name=\".u\" value=\"", 
		&ptr,
		"name=\".u\" value=\"%s\"");
	
	if(retVal != 0)
	{
		ptr    = NULL;
		retVal = u.SearchRegEx(resp.GetBodyPtr(), 
			"name=.u value=\"", 
			&ptr,
			"name=.u value=\"%s\"");
		
		if(retVal != 0)
		{
			/* Something's wrong! */
			logFile.Write(LOG_BASIC, "<%s> Login failed (cannot find .u variable!)", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
	}
		
	CStringEx challengeText;
	ptr    = NULL;
	retVal = challengeText.SearchRegEx(resp.GetBodyPtr(), 
		"name=\".challenge\" value=\"", 
		&ptr,
		"name=\".challenge\" value=\"%s\"");
	
	if(retVal != 0)
	{
		ptr    = NULL;
		retVal = challengeText.SearchRegEx(resp.GetBodyPtr(), 
			"name=.challenge value=\"", 
			&ptr,
			"name=.challenge value=\"%s\"");
		
		if(retVal != 0)
		{
			/* Something's wrong! */
			logFile.Write(LOG_BASIC, "<%s> Login failed (cannot find challenge!)", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
	}
		
	if(FALSE /* settings.m_bUseMD5 */)
	{
		if(TRUE /* settings.m_bSecureMode */)
			loginUrl = yahoo[HTTPS_LOGIN];
		else
			loginUrl = yahoo[HTTP_LOGIN];

		logFile.Write(LOG_ADVANCED, "<%s> Using MD5 secured password transmission", LPCTSTR(username));
		
		/*
		 * The following implementation of the hashing technique is 
		 * as per Yahoo's implementation which is available in the
		 * Javascript code at http://mail.yahoo.com/
		 */
		
		size_t newlen = 256 + (challengeText.length()*4) + 1; // arbitrary, but should suffice
		char *tmphash = (char *)malloc(newlen);
		
		MD5_CTX ctx;
		MD5_Init(&ctx);
		MD5_Update(&ctx, password, strlen(password));
		MD5_Final(reinterpret_cast<unsigned char *>(tmphash), &ctx);
		
		*tmphash = '\0';
		rhex(ctx.A, tmphash);
		rhex(ctx.B, tmphash);
		rhex(ctx.C, tmphash);
		rhex(ctx.D, tmphash);
		strcat(tmphash, challengeText);
		
		MD5_Init(&ctx);
		MD5_Update(&ctx, tmphash, strlen(tmphash));
		MD5_Final(reinterpret_cast<unsigned char *>(tmphash), &ctx);
		
		*tmphash = '\0';
		rhex(ctx.A, tmphash);
		rhex(ctx.B, tmphash);
		rhex(ctx.C, tmphash);
		rhex(ctx.D, tmphash);

		CStringEx hash(tmphash);
		free(tmphash); // we don't want the headaches of freeing this all over the place below
		
		c.Reset();
		resp.Reset();
		
		data.Format(".tries=1&.src=ym&.us=ym&.intl=us&.u=%s&login=%s&passwd=%s&.hash=1&.md5=1&.js=1&.challenge=%s&.persistent=y",
					LPCTSTR(u), LPCTSTR(UrlEncode(user)), LPCTSTR(hash), LPCTSTR(challengeText));
	}
	else
	{
		/* Normal connect without MD5 password hash */
		data.Format(".tries=1&.src=ym&.intl=us&.u=%s&login=%s&passwd=%s&.challenge=%s&.persistent=y&.chkP=Y&.done=%s&.v=0&hasMsgr=0&.save=%s&.js=&.md5=&.hash=&.js=&.last=&promo=&.bypass=&.partner=&.yplus=&.emailCode=&pkg=&stepid=&.ev=&.branch=",
				    LPCTSTR(u), LPCTSTR(UrlEncode(user)), LPCTSTR(UrlEncode(password)), LPCTSTR(challengeText), LPCTSTR(UrlEncode(mailurl)), LPCTSTR(UrlEncode("Sign In")));
		
		//if(settings.m_bSecureMode)
		//	loginUrl = yahoo[HTTPS_LOGIN];
		//else
		//	loginUrl = yahoo[HTTP_LOGIN];
	}

	logFile.Write(LOG_ADVANCED, "<%s> Login URL=%s, DATA=%s", LPCTSTR(username), LPCTSTR(loginUrl), LPCTSTR(MaskPwd(data)));
	if(c.DoPost(LPCTSTR(loginUrl), data, resp) != 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
		Unlock();
		return WEB_LOGIN_FAILED;
	}
	
	DumpPage("login",resp.GetBodyPtr());
	//START DEBUG
	//char *str = LoadPage("login");
	//resp.SetData(str, strlen(str));
	//free(str);
	// END DEBUG

	c.GetHost(host);

	CStringEx curfile = "login";
	
	if (resp.DataContains(".secdata"))   // Login - Verify
	{
#ifdef WIN32
		/* Extract .secdata */
		ptr = NULL;
		CStringEx secdata;
		// If secdata extraction fails, bail out
		if (secdata.SearchRegEx(resp.GetBodyPtr(), "name=\".secdata\" value=\".*\">", &ptr,
			"name=\".secdata\" value=\"%s\"") != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Unable to log in. Could not analyze login page (.secdata not found)", LPCTSTR(username));
			return WEB_LOGIN_FAILED;
		}
		logFile.Write(LOG_ADVANCED, "<%s> secdata = %s", LPCTSTR(username), LPCTSTR(secdata));

		/* Extract image */
		ptr = NULL;
		CStringEx img;
		CStringEx findimgurl;
		findimgurl.Format("img src=\"http[s]*://[a-zA-Z./]*%s.jpg\"", LPCTSTR(secdata));
		// If image extraction fails, bail out
		if (img.SearchRegEx(resp.GetBodyPtr(), findimgurl, &ptr,
			"img src=\"%s\"") != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Unable to log in. Could not analyze login page (image not found)", LPCTSTR(username));
			return WEB_LOGIN_FAILED;
		}
		logFile.Write(LOG_ADVANCED, "<%s> IMG = %s", LPCTSTR(username), LPCTSTR(img));

		CSendVerify dlg;
		dlg.setTitle("YPOPs! Login - Verify");
		dlg.m_url = img;
		int dr = dlg.DoModal();
		if(dr == IDOK)
		{
			CStringEx datatmp;
			//re-send with IMG and WORD (code)
			datatmp.Format(".secdata=%s&.secword=%s&%s",
				LPCTSTR(secdata), LPCTSTR(dlg.m_code), LPCTSTR(data));
			data = datatmp;
			c.Reset();
			resp.Reset();
			
			logFile.Write(LOG_ADVANCED, "<%s> secdata Login URL=%s, DATA=%s", LPCTSTR(username), LPCTSTR(loginUrl), LPCTSTR(MaskPwd(data)));
			if(c.DoPost(loginUrl.c_str(), data, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
				Unlock();
				return WEB_LOGIN_FAILED;
			}
			DumpPage(curfile = "login_secword", resp.GetBodyPtr());
		}
#else
		logFile.Write(LOG_BASIC, 
			"<%s> Yahoo requires user to log in and respond to the login challenge",
			LPCTSTR(username));
		Unlock();
		return WEB_LOGIN_FAILED;
#endif
	}

	/* Check if the login was successful */
	if(resp.DataContains("Invalid Password"))
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed due to invalid password", LPCTSTR(username));
		CheckError(curfile, resp);
		Unlock();
		return WEB_WRONG_PASSWORD;
	}
	
	if(resp.DataContains("ID does not exist"))
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed due to invalid user id", LPCTSTR(username));
		CheckError(curfile, resp);
		Unlock();
		return WEB_WRONG_USERNAME;
	}
	
	if (int r = CheckError(curfile, resp))
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
		return WEB_LOGIN_FAILED;
	}

	logFile.Write(LOG_ADVANCED, "<%s> host = %s", LPCTSTR(username), host.c_str());
	/* redirect to the first link if still on login page 
	 - this happens in secure mode */
	if(host == loginhost)
	{
		//logFile.Write(LOG_ADVANCED, "<%s> host is login.yahoo.com", LPCTSTR(username));
		/* Extract URL from return value */
		char *endPtr;
		char *startPtr = resp.GetBodyPtr();
		CStringEx url;
		char tempHost[1024];

		/* addition by K.Shyam for a nasty bug !.
		 * we assumed that the first url in this page is the login url
		 * but it is not so.so search till we get the login url
		 */
		while ( 1 )
		{
			endPtr = NULL;
			if ( url.SearchRegEx(startPtr, "http://", &endPtr, "%s\"") != 0 )
				break;/*we didnt have the regex inside */
			//logFile.Write(LOG_ADVANCED, "<%s> checking URL=%s", LPCTSTR(username), url.c_str());
			startPtr = endPtr + 7;/* move out of http:// */
			c.SplitUrl(url,tempHost);
			//logFile.Write(LOG_ADVANCED, "<%s> tempHost=%s", LPCTSTR(username), tempHost);
			if ( strstr(tempHost,mailhost) != NULL )
				break;
		}
		
		resp.Reset();
		c.Reset();
		logFile.Write(LOG_ADVANCED, "<%s> Redirected URL=%s", LPCTSTR(username), LPCTSTR(url));
		if(c.DoGet(url, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		DumpPage("login2",resp.GetBodyPtr());
		c.GetHost(host);
	}

	if(resp.DataContains("action='beta_welcome'"))
	{
		// user being asked to try new beta - user must refuse it manually
		logFile.Write(LOG_BASIC, "<%s> Yahoo is asking if you want to try the new Beta version of Yahoo! Mail", LPCTSTR(username));
		logFile.Write(LOG_BASIC, "<%s> YPOPs! does not yet work with the new beta.", LPCTSTR(username));
		//logFile.Write(LOG_BASIC, "<%s> If you can spare this account for maybe a week or so, please contact dbeusee@yahoo.com with login information", LPCTSTR(username));
		logFile.Write(LOG_BASIC, "<%s> You must log into Yahoo and press the \"No Thanks\" button - then ypops should work.", LPCTSTR(username));
		CStringEx betaHtmlStr;
		betaHtmlStr.Format("<html<body>\n\
YPOPs! cannot complete the login process for your \"<b>%s</b>\" account.<br>\n\
Yahoo is asking if you want to use the new Beta version of Yahoo! Mail.<p>\n\
YPOPs! does not yet work with the new beta.  \n\
You must log into this account using the Yahoo web site and press the \"No Thanks\"\n\
 button - after that, YPOPs! should work again with this account.<p>\n\
</body></html>\n", LPCTSTR(username));
		YahooError(betaHtmlStr);
		Unlock();
		return WEB_LOGIN_FAILED;
	}

#define YAHOO_BETA_NOT_SUPP

#ifdef YAHOO_BETA_OK
	ptr = NULL;
	CStringEx betaURL;
	// If image extraction fails, bail out
	if (betaURL.SearchRegEx(resp.GetBodyPtr(), "http://[^/]*/dc/[^']*", &ptr,
		"%s'") == 0)
	{
		logFile.Write(LOG_ADVANCED, "<%s> Beta URL = %s", LPCTSTR(username), LPCTSTR(betaURL));
		resp.Reset();
		c.Reset();
		if(c.DoGet(betaURL, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Beta Login failed", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		DumpPage("betalogin",resp.GetBodyPtr());
	}
	else
		logFile.Write(LOG_ADVANCED, "not using Beta", LPCTSTR(username));

#endif // YAHOO_BETA_OK

#if defined(YAHOO_BETA_NOT_SUPP) || defined(YAHOO_BETA_NOJAVA)
	if(resp.DataContains("Please enable Javascript"))
	{
#ifdef YAHOO_BETA_NOT_SUPP
#ifdef WIN32
		CStringEx html2;
		html2.Format("\
			<html><body>\
			<p>Yahoo Beta is not supported - you must stop using Yahoo Beta to use YPOPs!. \
			Please see <a href=http://help.yahoo.com/mail/ymail/gettingit/gettingit-05.html>Can \
			I switch between Yahoo! Mail and Yahoo! Mail Beta?</a> \
			for more information.<p>\n\
			</body></html>"
			);
		DumpPage("yahooerror", LPCTSTR(html2));

		CString logdir = GetLogDir();

		CYahooErrorDlg dlg2;
		dlg2.m_url.Format("file://%s/%s.yahooerror.html", LPCTSTR(logdir), LPCTSTR(username));
		int dr = dlg2.DoModal();
#endif // WIN32
		logFile.Write(LOG_BASIC, "<%s> Login failed - Yahoo Beta not supported", LPCTSTR(username));
		logFile.Write(LOG_BASIC, "<%s> Please see http://help.yahoo.com/mail/ymail/gettingit/gettingit-05.html", LPCTSTR(username));
		logFile.Write(LOG_BASIC, "<%s> for instructions on switching back", LPCTSTR(username));
		Unlock();
		return WEB_LOGIN_FAILED;
#endif // YAHOO_BETA_NOT_SUPP
#ifdef YAHOO_BETA_NOJAVA
		CStringEx url;
		CStringEx div;

		/* look for the following for beta:
		<div align="left" class='primaryHeading'>launch_php_js_1  
			<div class='primaryBodyText'>Please enable Javascript for your browser, or you can proceed to a 
<a href='/ym/BetaSwitcher?YY=&switch=1&src=beta&dest=current&nof=1'>non-JavaScript 
version of Yahoo! Mail</a> with limited functionality.</div></div>
		 */
		if ((!div.split(resp.GetBodyPtr(), "Please enable Javascript", ">non-JavaScript"))
		 || (!url.split(LPCTSTR(div), "a href='", "'")))
		{
			logFile.Write(LOG_BASIC, "<%s> Could not extract non-javascript url", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		url.Format("http://%s%s", LPCTSTR(host), LPCTSTR(url));
		resp.Reset();
		c.Reset();
		logFile.Write(LOG_ADVANCED, "<%s> non-javascript Redirected URL=%s", LPCTSTR(username), LPCTSTR(url));
		if(c.DoGet(url, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		DumpPage("login3",resp.GetBodyPtr());

		/* extract 2nd URL from here:
		<div align='left' class='subHeading'>After enabling Javascript...</div>
		<div align='left' class='primaryBodyText'>you can <a class="" href="http://mail.yahoo.com">return to the Yahoo! Mail Beta log-in page.</a></div>
		<div align='left' class='subHeading'>or...</div>
		<div align='left' class='primaryBodyText'>you can <a href="/ym/login?ymv=0&YY=4522934">proceed directly to original Yahoo! Mail</a></div>
		*/
		if ((!div.split(resp.GetBodyPtr(), "or...", "proceed directly to original"))
		 || (!url.split(LPCTSTR(div), "a href=\"", "\">")))
		{
			logFile.Write(LOG_BASIC, "<%s> Could not extract non-javascript final url",
				LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		url.Format("http://%s%s", LPCTSTR(host), LPCTSTR(url));
		
		logFile.Write(LOG_ADVANCED, "<%s> non-javascript Redirected URL #2=%s", LPCTSTR(username), LPCTSTR(url));
		if(c.DoPost(url, data, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		DumpPage("login4",resp.GetBodyPtr());
		c.GetHost(host);
#endif // YAHOO_BETA_NOJAVA
	}
#endif // defined(YAHOO_BETA_NOT_SUPP) || defined(YAHOO_BETA_NOJAVA)

	logFile.Write(LOG_ADVANCED, "<%s> Connected to %s.", LPCTSTR(username), LPCTSTR(host));

	/* host value should be in mail.<yahoo domain> domain */
    CStringEx mdomain;
    mdomain.Format(".%s", LPCTSTR(mailhost));
	if(host.Find(mdomain) < 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Login failed. No redirection to *%s.", LPCTSTR(username), LPCTSTR(mdomain));
		Unlock();
		return WEB_UNKNOWN_ERROR;
	}

	if(resp.DataContains("gWelcomePage"))
	{
		CStringEx url;
		CStringEx urlInbox;
		ptr = NULL;
		retVal = url.SearchRegEx(resp.GetBodyPtr(), 
			"var gWelcomePage = \".*\"", &ptr,
			"var gWelcomePage = \"%s\"");

		retVal = urlInbox.SearchRegEx(resp.GetBodyPtr(), 
			"var gInboxPage = \".*\"", &ptr,
			"var gInboxPage = \"%s\"");

		resp.Reset();
		c.Reset();
		logFile.Write(LOG_ADVANCED, "<%s> Login3 URL=%s", LPCTSTR(username), LPCTSTR(url));
		if(c.DoGet(url, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
			Unlock();
			return WEB_LOGIN_FAILED;
		}
		if (resp.DataContains(YAHOO_ERROR_SESSION_EXPIRED))
		{
			logFile.Write(LOG_ADVANCED, "<%s> got expired session error - initializing cUrl session",
				LPCTSTR(username), LPCTSTR(urlInbox));
			resp.Reset();
			c.FullReset();
			if(c.DoGet(url, resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> Login failed", LPCTSTR(username));
				Unlock();
				return WEB_LOGIN_FAILED;
			}
		}

		DumpPage("login3",resp.GetBodyPtr());
	}

	/* Extract the crumb value */
	ptr = NULL;
	CStringEx crumb;
	retVal = crumb.SearchRegEx(resp.GetBodyPtr(), 
		"=1&.crumb=.+&reset=1&YY=", &ptr,
		"=1&.crumb=%s&reset=1&YY=");

	if(retVal != 0)
	{
		ptr = NULL;
		retVal = crumb.SearchRegEx(resp.GetBodyPtr(), 
			"=1&.crumb=.+&", &ptr,
			"=1&.crumb=%s&");
	}

#ifdef YAHOO_BETA_OK
	if(retVal != 0)
	{
		// try beta mail format
		ptr = NULL;
		retVal = crumb.SearchRegEx(resp.GetBodyPtr(), 
			"wssid : '[^']*'", &ptr,
			"wssid : '%s'");
	}
#endif // YAHOO_BETA_OK

	if(retVal == 0)
	{
		crumbValue = crumb;
				
		logFile.Write(LOG_ADVANCED, "<%s> Crumb extracted: %s",
			LPCTSTR(username), LPCTSTR(crumb));
	}
	else
	{
		logFile.Write(LOG_BASIC, "<%s> Could not extract crumb", LPCTSTR(username));
		DumpPage("nocrumb",resp.GetBodyPtr());
		Unlock();
		return WEB_LOGIN_FAILED;
	}

	sessionCache.AddSession(username, password, crumbValue, session, host);

	logFile.Write(LOG_BASIC, "<%s> Login to Yahoo Mail succeeded.", LPCTSTR(username));

	Unlock(); // Other thread may now attempt to access section

	return WEB_LOGIN_OK;
}

/*******************************************************************
* Function  : FetchInboxList()
* Parameters: mailbox - The name of the mailbox (INBOX, BulkMail)
* Returns   : WEB_INBOX_FETCH_FAILED - Unable to retrieve email list
*             WEB_INBOX_EMPTY        - No emails in Inbox :(
*             WEB_INBOX_FETCH_OK     - Success :)
* Purpose   : To determine the number of emails in the Inbox and
*             to make the URL list for each email so that it can
*             be downloaded.
*******************************************************************/
int CWebBrowser::FetchMailboxList(const CStdString &folderName)
{
	int	  page		     = 0;
	char *ptr			 = NULL;
	int	  retVal		 = 0;
	char  inboxUrl[1024];
	char  folderPage[1024];
	unsigned int emailCtr = 0;
	
	HttpClient c(session);
	HttpClientResponse resp;
	
	/* Loop through the pages of the folder */
	while (!settings.m_bApplyLimit || (msgCount < settings.m_nMaxEmails))
	{
		sprintf(inboxUrl, yahoo[MBOX], LPCTSTR(host), LPCTSTR(UrlEncode(folderName)), page, settings.m_nEmailCategory);
		logFile.Write(LOG_ADVANCED, "<%s> inbox URL=%s", LPCTSTR(username), LPCTSTR(inboxUrl));
		if(c.DoGet(inboxUrl, resp) != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Error retrieving %s - fetch failed",
				LPCTSTR(username), LPCTSTR(folderName));
			return WEB_INBOX_FETCH_FAILED;
		}

		/* dump page for troubleshooting */
		sprintf(folderPage, "%s_page%0d", LPCTSTR(folderName), page);
		DumpPage(folderPage, resp.GetBodyPtr());

		if (CheckError(folderPage, resp))
		{
			logFile.Write(LOG_BASIC, "<%s> Yahoo Error retrieving %s - fetch failed",
				LPCTSTR(username), LPCTSTR(folderName));
			return WEB_INBOX_FETCH_FAILED;
		}

		/* A section dealing with locale parsing
		   has been removed (by ltD) */

		/* Extract all the message Id's from the page */
		CStringEx msgId;
		CStringEx newoldmark;
		MESSAGE* msgPtr  = NULL;
		MESSAGE* msgTemp = NULL; 
		char *ptr2		 = '\0';
		BOOL bFound		 = FALSE;

		/* Search for mailbox usage.  It looks like this as of 1/8/2005:
		<span id="quotausagebar">
		<span class="first" style="width:0%;"></span>
		<span class="last">0% of 250.0MB </span>
		                   \___________/________________this is what we want to get
		</span>
		*/

		ptr = NULL;
		CStringEx boxUsage;
		retVal = boxUsage.SearchRegEx(resp.GetBodyPtr(), 
		        "<span id=\"quotausagebar\">.*class=\"last\">[0-9]+\\% of [0-9.]+MB", &ptr, 
				"<span id=\"quotausagebar\">%*[^>]>%*[^=]=\"last\">%[^M]MB");
		if (retVal == 0 && boxUsage.length() > 0)
			boxUsage += "MB";
		else
		{
			ptr = NULL;
			retVal = boxUsage.SearchRegEx(resp.GetBodyPtr(), 
					"<span class=\"first\" style=\"width:[0-9]+", &ptr, 
					"<span class=\"first\" style=\"width:%s%%");
			if (retVal)
				boxUsage = "0%"; // no messages probably
			else if (boxUsage.length() > 0)
				boxUsage += "%";
		}
		if (boxUsage.length() == 0)
			logFile.Write(LOG_BASIC, "<%s> could not get mailbox usage", LPCTSTR(username));
		else
			logFile.Write(LOG_BASIC, "<%s> You use %s of your Yahoo! mailbox", LPCTSTR(username), boxUsage.c_str());
		
		ptr = resp.GetBodyPtr();

		/* Get id's from the current page */
		do
		{

			ptr++;

			retVal = newoldmark.SearchRegEx(resp.GetBodyPtr(), 
				        "<tr class=msg", &ptr, "<tr class=%s");

			BOOL bMsgNew=(retVal==0) && (strncmp(newoldmark,"msgnew",6)==0);

			retVal = msgId.SearchRegEx(resp.GetBodyPtr(), 
				"type=\"checkbox\" name=\"Mid\" value=\"", &ptr, 
				"type=\"checkbox\" name=\"Mid\" value=\"%s\"");
				
			if(retVal != 0)
			{
				retVal = msgId.SearchRegEx(resp.GetBodyPtr(), 
					"type=checkbox name=\"Mid\" value=\"", &ptr, 
					"type=checkbox name=\"Mid\" value=\"%s\"");
			}

			if(retVal == 0)
			{
				/* 
				* Check if this message already exists
				* in the list. If not, then add it.
				*/
				msgTemp = msgHead;
				bFound  = FALSE;
				
				while(msgTemp != NULL)
				{
					if(msgTemp->msgId == msgId)
					{
						bFound = TRUE;
						break;
					}
					
					msgTemp = msgTemp->next;
				}
				
				if(bFound)
					continue;
				
				msgPtr = new MESSAGE;
				msgPtr->next = NULL;
				msgPtr->msgId = msgId;
				msgPtr->mailbox = folderName;
				msgPtr->email.SetTempSize(1024);
				msgPtr->email.SetUnread(bMsgNew);
					
				/* message id is a sequence of numbers
				separated by underscore - second and third
				of those may change so remove them
				for UIDL purposes
				(experimental change by ltd) */
				int hypcount=0;
				CStdString msgshortid;
				for (int i=0;msgPtr->msgId[i]!=0;i++) {
					if (msgPtr->msgId[i]=='_') hypcount++;
					if (hypcount>=9) break;
					if ((hypcount!=1) && (hypcount!=2)) msgshortid+=msgPtr->msgId[i];
				}
				msgPtr->email.SetMsgId(msgshortid);
				
				if(msgHead == NULL)
				{
					msgHead = msgPtr;
					msgTail = msgPtr;
				}
				else
				{
					msgTail->next = msgPtr;
					msgTail = msgPtr;
				}
				
				msgCount++;
				emailCtr++;
			}
			
			if((msgCount >= settings.m_nMaxEmails) && settings.m_bApplyLimit)
			{
			/* 
			* We cannot download any more emails for
			* this session as we have hit the max
			* limit.
				*/
				logFile.Write(LOG_ADVANCED, "<%s> Message limit (%d) for session reached", LPCTSTR(username), settings.m_nMaxEmails);
				break;
			}
		} while(retVal == 0);
		
		/* Dump the folder page for debugging
		   if no id's were extracted */
		if (!emailCtr) {
			logFile.Write(LOG_BASIC, "<%s> No messages were found. Folder probably empty", LPCTSTR(username));
			/* dump page for troubleshooting */
			sprintf(folderPage, "%s_0msgs", LPCTSTR(folderName));
			DumpPage(folderPage, resp.GetBodyPtr());
		}

		/* Scan the page for a part of a link to the next one */
		ptr = strstr(resp.GetBodyPtr(), "&next=1&");
		
		/* Don't download the next page if limit has been hit
		   or there is no next page */
		if((ptr==NULL) || (settings.m_bApplyLimit && (msgCount >= settings.m_nMaxEmails)))
			break;

		page++;
		
		c.Reset();
		resp.Reset();
	}
	
	logFile.Write(LOG_ADVANCED, "<%s> Extracted %d message Id's from mailbox %s", LPCTSTR(username), emailCtr, LPCTSTR(folderName));
	
	return WEB_INBOX_FETCH_OK;
}

/*******************************************************************
* Function  : GetMsgCount()
* Parameters: - 
* Returns   : The number of emails in the mailbox
* Purpose   : To return the count of the number of emails in the
*             users mailbox.
*******************************************************************/
int CWebBrowser::GetMsgCount()
{
	return msgCount;
}

/*******************************************************************
* Function  : DownloadAllEmails()
* Parameters: progress - Reference to the progress bar dialog
* Returns   : 0
* Purpose   : To download each and every email in the Yahoo 
*             mailbox
*******************************************************************/
#ifdef WIN32
int CWebBrowser::DownloadAllEmails(CProgressWnd *progress)
#else
int CWebBrowser::DownloadAllEmails()
#endif
{
	CStdString		   progressStr;
	MESSAGE			   *msg = msgHead;
	int				   ctr = 1;
	
	while(msg != NULL)
	{
#ifdef WIN32
		if(progress != NULL)
		{
			progressStr.Format("Downloading email %d of %d", ctr, GetMsgCount());
			progress->SetText(progressStr);
			progress->PeekAndPump();
		}
#endif
		logFile.Write(LOG_BASIC, "<%s> Downloading email %d", LPCTSTR(username), ctr);
		
		ctr++;
		
		DownloadEmail(msg);

		msg = msg->next;

#ifdef WIN32
		if(progress != NULL)
		{
			progress->StepIt();
			
			if(progress->Cancelled())
			{
				/* User clicked on the Cancel button */
				return 1;
			}
		}
#endif
	}
	
	return 0;
}

/*******************************************************************
* Function  : DownloadEmail()
* Parameters: msg - message to be downloaded
*             heasersonly - default FALSE
* Returns   : 0
* Purpose   : To download one message
*******************************************************************/
int CWebBrowser::DownloadEmail(MESSAGE *msg, BOOL headersonly)
{
	char			   *ptr = NULL;
	CStringEx		   url;
	HttpClient		   c(session);
	HttpClientResponse resp;
	int				   ok;
	
	/* Download the email header */
	if (!msg->email.IsHeaderDownloaded()) {

		/* return error if the limit of downloads has been reached */
		if (settings.m_bIgnoreSize && (downloadedCount>=settings.m_nMaxEmails)) 
			return 1;

		url.Format(yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), LPCTSTR(msg->msgId), "HEADER");
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), url.c_str());
	
		/* Edit SH. Make sure content-type contains 'filename' to check that 
		the request was processed correctly */
		c.Reset();
		resp.Reset();
		ok = (c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename");
		if (!ok)
		{
			logFile.Write(LOG_ADVANCED, "<%s> failed - download trying again", LPCTSTR(username));
			sleep(1);
			c.FullReset();
			resp.Reset();
			ok = (c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename");
		}

		if (ok)
		{
		    msg->email.SetHeader(resp.GetBodyPtr());

			/* Add X-YPOPs-Folder header */
			msg->email.AddHeader("X-YPOPs-Folder", LPCTSTR(msg->mailbox));
	
			/* Increase the count of downloads*/
			downloadedCount++;
			DumpPage("messageheader", resp.GetBodyPtr());
		}
		else
		{
			/* The headers couldn't be downloaded so skip this email */
			logFile.Write(LOG_BASIC, "<%s> Header could not be downloaded", LPCTSTR(username));
			DumpPage("messageheader", resp.GetBodyPtr());
			return 1;
		}
	}

	c.Reset();
	resp.Reset();

	/* Download the email body (including attachments) */
	if (!headersonly) {
		url.Format(yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), LPCTSTR(msg->msgId), "TEXT");
	
		/* SH. Used for testing invalid msgs/bandwidth over quota etc
		// url.Format(yahoo[MSG], LPCTSTR(host), LPCTSTR(UrlEncode(msg->mailbox)), "BUNCHO_CRAP", "TEXT");
		*/
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), url.c_str());
		
		ok = (c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename");
		if (!ok)
		{
			logFile.Write(LOG_ADVANCED, "<%s> failed - download trying again", LPCTSTR(username));
			sleep(1);
			c.FullReset();
			resp.Reset();
			ok = (c.DoGet(url, resp) == 0) && resp.ContentTypeContains("filename");
		}
		
		if (ok)
		{
			/* if email body is empty */
			if (resp.GetBodyPtr()==NULL) {
				msg->email.SetBody("");
				logFile.Write(LOG_ADVANCED, "<%s> message bas no body", LPCTSTR(username));
			} else if (msg->email.GetHeader("Content-Transfer-Encoding").ToLower()=="base64") {
				DumpPage("messagebody", resp.GetBodyPtr());
				/* if body was encoded by sender (Yahoo has decoded it), encode it again */
				DwString dwbody;
				dwbody.assign(resp.GetBodyPtr(),resp.GetBodyLength());
				DwEncodeBase64(dwbody,dwbody);
				msg->email.SetBody(dwbody.c_str());
				logFile.Write(LOG_ADVANCED, "<%s> re-encoded base64 message", LPCTSTR(username));
			} else {
				DumpPage("messagebody", resp.GetBodyPtr());
				// replace LF with CRLF and double '.'.  We don't use CStdString's Replace function because
				// it doesn't perform well for this kind of use.

				/* Loop 1:Count occurences of 'LF' and lines begining with '.' */
				char *ptr=resp.GetBodyPtr();
				unsigned long lfcount=0,dotcount=0;
				if (*ptr=='.') dotcount++;	//the first line begins with '.'
				for(;*ptr!=0;ptr++)
					if (*ptr=='\n') {
						lfcount++;
						if (*(ptr+1)=='.') dotcount++;
					}

				/* Check whether email in text format */
				BOOL bTextPlain=msg->email.GetHeader("Content-Type").Left(10).ToLower()=="text/plain";

				/* Reserve memory for the new body: current size of the body+
				   +number of lines (only if not text email)+
				   +number of lines beginning with '.'+trailing '\0' */
				int datasz = resp.GetBodyLength()+(bTextPlain?0:lfcount)+dotcount+1;
				char *data = new char[datasz];

				logFile.Write(LOG_ADVANCED, "<%s> message body size is %d, len=%d, content-type=%s",
							  LPCTSTR(username), datasz, strlen(resp.GetBodyPtr()),
							  LPCTSTR(msg->email.GetHeader("Content-Transfer-Encoding")));

				if (data != NULL && (lfcount > 0 || dotcount > 0)) {
					ptr = resp.GetBodyPtr();
					char *newptr = data;
					if (*ptr=='.') *newptr++='.';	//if the first line begins with '.'

					/* Loop 2:Replace LF with CRLF unless the email is in text format 
					   and double '.' at beginning of lines */
					for(; *ptr != 0; ptr++)
						if (*ptr == '\n') {			//in case of LF
							if (!bTextPlain) *newptr++ = '\r';			//add CR
							*newptr++='\n';
							if (*(ptr+1) == '.') *newptr++ = '.';	//double '.'
						}
						else
							*newptr++=*ptr;

					*newptr = 0;
					logFile.Write(LOG_ADVANCED, "<%s> setting message body after fixing text - size is %d, strlen=%d",
								  LPCTSTR(username), datasz, strlen(resp.GetBodyPtr()));
					msg->email.SetBody(data);
					delete [] data;
				} 
				else {
					if (data != NULL)
						delete [] data;
					else
						/* The memory could not be reserved */
						logFile.Write(LOG_ADVANCED, "<%s> WARNING: Memory allocation problem - using original message", LPCTSTR(username));
					logFile.Write(LOG_ADVANCED, "<%s> setting message body - size is %d", LPCTSTR(username), strlen(resp.GetBodyPtr()));
					msg->email.SetBody(resp.GetBodyPtr());
					logFile.Write(LOG_ADVANCED, "<%s> set message body - size is %d", LPCTSTR(username), msg->email.GetBodySize());
				}
			}
		}
		else
		{
			/* The email body could not be downloaded, so skip this message */
			logFile.Write(LOG_ADVANCED, "<%s> Email could not be downloaded", LPCTSTR(username));
			if (resp.GetBodyPtr() != NULL)
				DumpPage("messagebody", resp.GetBodyPtr());
			else
				logFile.Write(LOG_ADVANCED, "<%s> no messagebody to dump", LPCTSTR(username));
			return 1;
		}
	}

	return 0;
}

/*******************************************************************
* Function  : DownloadEmail()
* Parameters: msgNumber - message to be downloaded
*             heasersonly - default FALSE
* Returns   : 0
* Purpose   : Overloaded version finds the right pointer
			  and downloads one message
*******************************************************************/
int CWebBrowser::DownloadEmail(int msgNumber, BOOL headersonly)
{
	MESSAGE *ptr=msgHead;

	if(msgNumber > GetMsgCount())
		return 1;

	for (int i=1;i<msgNumber;i++) ptr=ptr->next;

	return DownloadEmail(ptr, headersonly);
}

/*******************************************************************
* Function  : GetMessage and GetEmail
* Parameters: msgNumber - The email to retrieve
* Returns   : Pointer to an MESSAGE or Email object respectively
*             or NULL if it could not be found
* Purpose   : To retrieve a specific email from the list
*******************************************************************/
MESSAGE* CWebBrowser::GetMessage(int msgNumber)
{
	MESSAGE *ptr  = msgHead;
	MESSAGE *prev = msgHead;
	
	if(msgNumber > GetMsgCount())
		/* Invalid */
		return NULL;
	
	for(int i = 0; i < msgNumber && ptr != NULL; i++)
	{
		prev = ptr;
		ptr  = ptr->next;
	}
	
	if (prev!=NULL) return prev;
	else return NULL;
}
Email* CWebBrowser::GetEmail(int msgNumber)
{
	MESSAGE *ptr  = GetMessage(msgNumber);
	if (ptr!=NULL) return &(ptr->email);
	else return NULL;
}

/*******************************************************************
* Function  : AnyEmailsToDelete()
* Parameters: -
* Returns   : true - there are emails to delete; false otherwise
* Purpose   : To determine if there any any emails to delete
*******************************************************************/
bool CWebBrowser::AnyEmailsToDelete()
{
	MESSAGE	*ptr = msgHead;

   	while(ptr != NULL)
	{
		if(ptr->email.IsDeleted())
		{
            return true;
		}
		
		ptr = ptr->next;
    }

    return false;
}

/*******************************************************************
* Function  : rhex
* Parameters: num - A number from the MD5 context (A, B, C or D)
* Returns   : A hex string with ls-byte first
* Purpose   : Convert a 32-bit number to a hex string with ls-byte 
*             first. 
*******************************************************************/
inline void CWebBrowser::rhex(unsigned int num, char *data)
{
	CStdString hex_chr = "0123456789abcdef";
	CStdString str;
	
	for(int j = 0; j <= 3; j++)
	{
		str += hex_chr[(num >> (j * 8 + 4)) & 0x0F];
		str += hex_chr[(num >> (j * 8)) & 0x0F];
	}
	
	strcat(data, LPCTSTR(str));
}

/*******************************************************************
* Function  : EmptyTrash
* Parameters: -
* Returns   : -
* Purpose   : Empty the Yahoo Trash folder
*******************************************************************/
void CWebBrowser::EmptyTrash()
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			trashUrl;
	
    if(GetMsgCount() > 0)
	{
		/* Empty trash only if some emails were downloaded */
		trashUrl.Format(yahoo[TRASH], LPCTSTR(host), LPCTSTR(crumbValue));
		
		logFile.Write(LOG_BASIC, "<%s> Emptying trash", LPCTSTR(username));
		logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(trashUrl));
		
		c.DoGet(trashUrl, resp);
	}
}

/*******************************************************************
* Function  : EmptyBulkMail
* Parameters: 
* Returns   : 
* Purpose   : Empty the Yahoo Bulk folder
*******************************************************************/
void CWebBrowser::EmptyBulkMail()
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			trashUrl;
	
    trashUrl.Format(yahoo[EMPTY_BULKMAIL], LPCTSTR(host), LPCTSTR(crumbValue));
	
	logFile.Write(LOG_BASIC, "<%s> Emptying bulk mail folder", LPCTSTR(username));
	logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(trashUrl));
	
	c.DoGet(trashUrl, resp);
}

/*******************************************************************
* Function  : LogOff
* Parameters: -
* Returns   : -
* Purpose   : Log off Yahoo! service
*******************************************************************/
void CWebBrowser::LogOff()
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			url;
	
    url.Format(yahoo[LOGOFF], LPCTSTR(host));
	
	logFile.Write(LOG_BASIC, "<%s> Logging off", LPCTSTR(username));
	logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(url));
	
	c.DoGet(url, resp);

}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::MarkUnread(const CStdString &mailbox)
{
	
	/* Delete all the emails now. */
	MESSAGE				*ptr = msgHead;
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStdString			url;
	
	int ctr = 0;
	
	url.Format(yahoo[UNREAD], LPCTSTR(host), LPCTSTR(mailbox), LPCTSTR(crumbValue));
	
	while(ptr != NULL)
	{
		if(ptr->email.IsHeaderDownloaded() && ptr->email.IsUnread()
			&& !ptr->email.IsDeleted() && ptr->mailbox == mailbox)
		{
			url += "&Mid=";
			url += ptr->msgId;
			ctr++;
		}
		
		ptr = ptr->next;
		
		if((ctr==5) || (ctr && (ptr==NULL)))
		{		
			c.DoGet(url, resp);
			c.Reset();
			resp.Reset();
			
			logFile.Write(LOG_ADVANCED, "<%s> Marking messages as unread (Mailbox = %s)", LPCTSTR(username), LPCTSTR(mailbox));
			logFile.Write(LOG_ADVANCED, "<%s> URL: %s", LPCTSTR(username), LPCTSTR(url));
			
			ctr = 0;
			url.Format(yahoo[UNREAD], LPCTSTR(host), LPCTSTR(mailbox), LPCTSTR(crumbValue));
		}
	}
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::SetSession(YAHOO_SESSION *ySession)
{
	this->session = ySession->session;
	this->host = ySession->host;
	this->username = ySession->username;
	this->password = ySession->password;
	this->crumbValue = ySession->crumbValue;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::SaveSession()
{
	sessionCache.AddSession(username, password, crumbValue, session, host);
}

/*******************************************************************
* Function  : SendMail
* Parameters: to, cc, bcc - email recipients, subject - email title
			  body - email contents, htmlbody - set if rich text format
			  EmailAttachmentList - mimepp list of attachments
* Returns   : 0 - success, 1 - retrieving of compose page failed
			  2 - email submission failed, 3 - attachment submission failed
* Purpose   : To submit an email message to Yahoo! for sending
*******************************************************************/
int CWebBrowser::SendEmail(CStdString &to, CStdString &cc, 
						   CStdString &bcc, CStdString &subject, 
						   CStdString &body, bool htmlbody,
						   const EmailAttachmentList &attachment)
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStringEx			url;
	CStringEx			postData;
	CStringEx			contentLength;
	char				*ptr = NULL;
	CStringEx			crumb;
	CStringEx			NumAtt("0");
	char				*AttData = NULL;
	int					retVal;
	CStringEx			yy;
	
	/* Get the Compose page */
	url.Format(yahoo[COMPOSE_GET], LPCTSTR(host));

	if(c.DoGet(url, resp) != 0) {
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not download compose page", LPCTSTR(username), LPCTSTR(to));
		return 1;
	}
	
	/* Extract crumb */
	retVal = crumb.SearchRegEx(resp.GetBodyPtr(), 
		"name=\".crumb\" value=\"", &ptr,
		"name=\".crumb\" value=\"%s\"");

	/* Extract YY */
	ptr = NULL;
	/* If crumb or YY extraction has failed dump the compose page to a file */
	if (yy.SearchRegEx(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s\"") != 0
		|| retVal!=0) {
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not analyze compose page (YY= not found)", LPCTSTR(username), LPCTSTR(to));
		/* Dump the compose page */
		DumpPage("compose", resp.GetBodyPtr());
		return 1;
	}

	/* Send attachments (if any) */
	if(attachment.Count() > 0)
	{
		/* prepare multipart form */
		struct curl_httppost *formpost=NULL;
		struct curl_httppost *lastptr=NULL;
		CStdString Formfiletype;
		CStdString Formfilename;
		CStdString PrevFilename = "";
		int reqnumatt=0;	//number of posted attachments

		logFile.Write(LOG_ADVANCED, "<%s> SendEmail() processing attachments...", username.c_str());
		/* iterate all requested attachments with retVal as counter */
		for(retVal = 0; retVal < attachment.Count() ; retVal++) {
			const EmailAttachment *att = &attachment.Get(retVal);
			/* if there's no filename it's not a real attachment.
			   For instance, mimepp treats html parts of rich text emails
			   as attachments.
			   Yahoo's web interface doesn't allow forwarding message attachments built from scratch.  It only
			   allows forwarding an email sent to the Yahoo account and still exists in the user's inbox.
			   The web interface does this by referencing the msgid in the attachment.  YPOPs can't do this
			   because the SMTP session has no knowledge of the msgId the user is responding to.  To get around
			   this problem, the user must tell Outlook to forward inline, not as attachment. */
			Formfiletype.Format("%s/%s", att->Type().c_str(), att->Subtype().c_str());
			logFile.Write(LOG_ADVANCED, "<%s> SendEmail() attachment type=%s", username.c_str(), Formfiletype.c_str());

			if (!att->FileName().empty()) {

				CStdString fileName = att->FileName();
				if (fileName.Equals(PrevFilename)) {
					int i = fileName.ReverseFind('.');
					if (i == -1)
						fileName.AppendFormat("_%d", retVal+1);
					else
						fileName.Format("%s_%d.%s",
							fileName.Left(i).c_str(),
							retVal+1,
							fileName.Mid(i+1, fileName.length()).c_str());
				}
				PrevFilename = att->FileName();

				/* Ugly trick here to include file name header 
				   in form field name header. */
				Formfilename.Format("userFile%d\"; filename=\"%s", retVal, fileName.c_str());

				/* add attchment to multipart form */
				curl_formadd(&formpost, &lastptr,
					CURLFORM_COPYNAME, Formfilename.c_str(),
					CURLFORM_CONTENTTYPE, Formfiletype.c_str(),
					CURLFORM_PTRCONTENTS, att->Content().c_str(), 
					CURLFORM_CONTENTSLENGTH, att->Size(),
					CURLFORM_END);

				reqnumatt++;
				logFile.Write(LOG_ADVANCED, "<%s> SendEmail() added attachment #%d (filename=%s)",
						username.c_str(), retVal+1, fileName.c_str());

			} //if
			else
			{
				if (attachment.Get(retVal).Type() == "message")
				{
					// TODO: put up dialog to inform user we can't send the message
					logFile.Write(LOG_BASIC, "<%s> SendEmail() ERROR - \"message\" type attachments cannot be processed (Yahoo! web interface restriction)", username.c_str());
					logFile.Write(LOG_BASIC, "<%s> SendEmail() change your email client to forward messages inline instead of forward as attachment", username.c_str());
				}
				else
				{
					logFile.Write(LOG_ADVANCED, "<%s> SendEmail() WARNING - skipping attachment (no filename)", username.c_str());
				}
			}
		} //for
		
		/* Proceed with attachments only when good ones have been found */
		if (reqnumatt) {
			/* add upload request to multipart form */
			curl_formadd(&formpost, &lastptr,
				CURLFORM_COPYNAME, "UPL",
				CURLFORM_COPYCONTENTS, "1", 
				CURLFORM_END);

			/* send the form */
			c.Reset();
			resp.Reset();
		
			url.Format(yahoo[COMPOSE_ATTACH], LPCTSTR(host), yy.c_str());
			logFile.Write(LOG_ADVANCED, "<%s> SendEmail() COMPOSE_ATTACH URL=%s", username.c_str(), url.c_str());

			/* the last parameter indicates posting multipart/form-data */
			if (c.DoPost(url, formpost, resp, true)!=0) {
				
				curl_formfree(formpost);
				
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not upload attachments", LPCTSTR(username), LPCTSTR(to));
				return 3;
			}
			
			curl_formfree(formpost);
			
			logFile.Write(LOG_ADVANCED, "<%s> SendEmail() extracting YY, NumAtt, and AttData", username.c_str());
			/* Extract YY */
			ptr = NULL;
			retVal = yy.SearchRegEx(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s\"");
			
			/* Extract NumAtt */
			ptr = NULL;
			retVal = NumAtt.SearchRegEx(resp.GetBodyPtr(), "name=NumAtt value=\"", &ptr, "name=NumAtt value=\"%s\"");
			
			/* Extract AttData */
			if (NULL != (ptr = strstr(resp.GetBodyPtr(), "name=AttData value=\""))) {
				/* Skip to the value of "value" */
				ptr +=sizeof("name=AttData value=\"") - 1;
				/* cut off the quote */
				retVal=strcspn(ptr, "\"");
				SAFE_FREE(AttData);
				AttData = (char *)malloc(retVal+1);
				strncpy(AttData, ptr, retVal);
				AttData[retVal]=0;
				/* Encode */
				CStdString ad = UrlEncode(AttData);
				free(AttData);
				AttData = (char *)malloc(ad.length()+1);
				strcpy(AttData, ad.c_str());
			}
			else
			{
				AttData = (char *)malloc(1);
				AttData[0] = TCSNULL;
			}
			logFile.Write(LOG_ADVANCED, "<%s> SendEmail() YY=%s, NumAtt=%s, and strlen(AttData)=%d",
				username.c_str(), yy.c_str(), NumAtt.c_str(), strlen(AttData));

			/* check the number of successfully uploaded attachments */
			if (retVal=(reqnumatt-atoi(NumAtt))) {
				
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. %d attchments not uploaded", LPCTSTR(username), LPCTSTR(to), retVal);

				/* Dump the attachment page */
				DumpPage("attachments", resp.GetBodyPtr());
					
				/* Send a cancel request */
				url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy.c_str());
				
				c.Reset();
				resp.Reset();
				
				postData.Format(".crumb=%s&NumAtt=%s&AttData=%s&CAN=Cancel", 
					crumb.c_str(), NumAtt.c_str(), AttData);
				
				contentLength.Format("%d", postData.GetLength());
				c.AddHeader("Content-Length", LPCTSTR(contentLength));

				/* Log a successful cancel request */
				if (c.DoPost(url, LPCTSTR(postData), resp)==0)
					logFile.Write(LOG_ADVANCED, "<%s> Attachments cancelled", LPCTSTR(username));
				
				SAFE_FREE(AttData);
				return 3;
			} //if (i=(reqnumatt-atoi(NumAtt)))
		} //if (reqnumatt)
	} //if(attachment.Count() > 0)

	logFile.Write(LOG_ADVANCED, "<%s> SendEmail() changing '..' to '.'", username.c_str());
	/* Edit SH - Change ..'s to .'s. */
	if (body[0]=='.') body.Delete(0);
	body.Replace("\r\n..", "\r\n.");
	
	/* Generate response page with email information */

	logFile.Write(LOG_ADVANCED, "<%s> SendEmail() building post data...", username.c_str());
	/* Yahoo does not accept empty To recipients so set it to "" */
	if (to.IsEmpty()) to="\"\"";
	char * escTo  = curl_escape(LPCTSTR(to), 0);
	char * escCc  = curl_escape(LPCTSTR(cc), 0);
	char * escBcc = curl_escape(LPCTSTR(bcc), 0);
	char * escSub = curl_escape(LPCTSTR(subject), 0);
	char * escBody= curl_escape(LPCTSTR(body), 0);

	postData.Format(".crumb=%s&To=%s&Cc=%s&Bcc=%s&Subj=%s&Body=", 
                    crumb.c_str(), escTo, escCc, escBcc, escSub);
    postData += escBody;
	postData += "&NumAtt=";
	postData += NumAtt;
	postData += "&AttData=";
	if (AttData != NULL) postData += AttData;

	postData += "&box=&FwdFile=&FwdMsg=&FwdSubj=&OriginalFrom=&OriginalSubject=&InReplyTo=&FName=&Video=&ATT=&VID=&Format=";
	postData += htmlbody ? "html" : "";
	postData += "&SigAtt=&SaveCopy=";
	postData += settings.m_bSaveEmailCopy ? "yes" : "no";
	postData += "&NoSubj=1&NoBody=1&SEND=%A0%A0Send%A0%A0\n";

	/* free data */
	curl_free(escTo);
	curl_free(escCc);
	curl_free(escBcc);
	curl_free(escSub);
	curl_free(escBody);

	url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy.c_str());

	c.Reset();
	resp.Reset();
	
	contentLength.Format("%d", postData.GetLength());
	c.AddHeader("Content-Length", LPCTSTR(contentLength));
	c.AddHeader("Content-Type", "application/x-www-form-urlencoded");

	logFile.Write(LOG_ADVANCED, "<%s> SendEmail() sending message...", username.c_str());
	/* send the message */
	if(c.DoPost(url, LPCTSTR(postData), resp) != 0)
	{
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not submit message", LPCTSTR(username));
		SAFE_FREE(AttData);
		return 2;
	}
	
	/* Dump the response page to a file so that it is easy to analyze it */

	CStringEx curfile;
	DumpPage(curfile = "sendresponse", resp.GetBodyPtr());
	//char *str = LoadPage("sendresponse");
	//resp.SetData(str, strlen(str));
	//free(str);


	if (resp.DataContains("ImgVerification"))   // Send - Verify
	{
#ifdef WIN32
		/* Extract IMG */
		ptr = NULL;
		CStringEx img;
		// If IMG extraction fails, bail out
		if (img.SearchRegEx(resp.GetBodyPtr(), "name=\"IMG\" value=\".*\">", &ptr,
			"name=\"IMG\" value=\"%s\"") != 0)
		{
			logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not analyze compose page (IMG not found)", LPCTSTR(username), LPCTSTR(to));
			return 1;
		}
		logFile.Write(LOG_ADVANCED, "<%s> IMG = %s", LPCTSTR(username), LPCTSTR(img));

		//CSendVerify dlg(::AfxGetMainWnd());
		CSendVerify dlg;
		dlg.m_url = img;
		int dr = dlg.DoModal();
		if(dr == IDOK)
		{
			//re-send with IMG and WORD (code)
			CStringEx postDataCh; 
			postDataCh.Format("IMG=%s&WORD=%s&%s", LPCTSTR(img), LPCTSTR(dlg.m_code), LPCTSTR(postData));
			c.Reset();
			resp.Reset();
			
			contentLength.Format("%d", postDataCh.GetLength());
			c.AddHeader("Content-Length", LPCTSTR(contentLength));
			c.AddHeader("Content-Type", "application/x-www-form-urlencoded");

			logFile.Write(LOG_ADVANCED, "<%s> SendEmail() sending message with WORD=\"%s\"...", LPCTSTR(username),LPCTSTR(dlg.m_code));
			/* send the message */
			if(c.DoPost(url, LPCTSTR(postDataCh), resp) != 0)
			{
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Could not submit message", LPCTSTR(username));
				SAFE_FREE(AttData);
				return 2;
			}
			DumpPage(curfile = "sendchallengeresponse", resp.GetBodyPtr());
			if (resp.DataContains("ImgVerification"))   // Send - Verify
			{
				logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Challenge not accepted", LPCTSTR(username));
				SAFE_FREE(AttData);
				return 2;
			}
		}
#else
		logFile.Write(LOG_BASIC, 
			"<%s> Yahoo requires user to log in and send a message and respond to the send verify challenge",
			LPCTSTR(username));
#endif
#ifdef WIN32
		if(dr != IDOK) {
#endif
			SAFE_FREE(AttData);
			return 2;
#ifdef WIN32
		}
#endif
	}

	if (int r = CheckError(curfile, resp))
	{
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s.", LPCTSTR(username), LPCTSTR(to));
		SAFE_FREE(AttData);
		return r;
	}

	/* Retrieved page should contain AddAddresses form even if
	   the address is already in the book */
	if (!resp.DataContains("name=\"AddAddresses\""))
	{
		logFile.Write(LOG_BASIC, "<%s> Unable to send email to %s. Yahoo! has not accepted the message", LPCTSTR(username), LPCTSTR(to));

		// TODO: put up a dialog box here that shows the .html file using some sort of HTML control
		// we could use an IE control since it would be windows specific

		/* Does the page contain YY ? */
		ptr = NULL;
		if (yy.SearchRegEx(resp.GetBodyPtr(), "YY=", &ptr, "YY=%s\"")==0)
		{
			/* Send a cancel request */
			url.Format(yahoo[COMPOSE_POST], LPCTSTR(host), yy.c_str());
		
			c.Reset();
			resp.Reset();
		
			postData.Format(".crumb=%s&NumAtt=%s&AttData=%s&CAN=Cancel", 
			crumb.c_str(), NumAtt.c_str(), nvl(AttData,""));
		
			contentLength.Format("%d", postData.GetLength());
			c.AddHeader("Content-Length", LPCTSTR(contentLength));
		
			/* Log a successful cancel request */
			if (c.DoPost(url, LPCTSTR(postData), resp)==0)
				logFile.Write(LOG_ADVANCED, "<%s> Message cancelled", LPCTSTR(username));
			else
				logFile.Write(LOG_ADVANCED, "<%s> Message not cancelled", LPCTSTR(username));
		}

		SAFE_FREE(AttData);
		return 2;
	}

	SAFE_FREE(AttData);
	return 0;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
CStdString CWebBrowser::UrlEncode(const char *data)
{
	char *escapedUrl = curl_escape(data, 0);
	CStdString url(escapedUrl);
	curl_free(escapedUrl);

	return url;
}

/*******************************************************************
* Function  : 
* Parameters: 
* Returns   : 
* Purpose   : 
*******************************************************************/
void CWebBrowser::DeleteSession()
{
	//Remove it from the cache.
	if (sessionCache.DeleteSession(username) == 0 && username != "")
    	delete session;
    session = NULL;
}

/*******************************************************************
* Function  : DumpPage
* Parameters: page_name, page_content
* Returns   : 
* Purpose   : If loggin is advanced, dumps page_content
			  to file <username>.<page_name>.html
*******************************************************************/
void CWebBrowser::DumpPage(const char* page_name, const char* page_content)
{
	if (settings.m_bCreateLog && settings.m_nLogLevel == LOG_ADVANCED)
	{            
		CStringEx filename;
		filename.Format("%s/%s.%s.html", LPCTSTR(GetLogDir()), LPCTSTR(username), page_name);
		FILE *fp = fopen(LPCTSTR(filename), "w");

		if(fp != NULL)
		{
			char *ptr = strstr(page_content, "<META HTTP-EQUIV=Refresh");
			if (ptr)
				*(ptr+1) = 'X'; // set so browser doesn't recognize refresh request
			fwrite(page_content, 1, strlen(page_content), fp);
			if (ptr) *(ptr+1) = 'M'; // set back to M
			fclose(fp);               
			logFile.Write(LOG_ADVANCED, "<%s> Page has been dumped to '%s' for analysis", LPCTSTR(username), LPCTSTR(filename));
		}
		else
		{
			logFile.Write(LOG_ADVANCED, "<%s> Dump file '%s' could not be created.", LPCTSTR(username), LPCTSTR(filename));
		}
	}
}

/*******************************************************************
* Function  : LoadPage
* Parameters: page_name, page_content
* Returns   : 
* Purpose   : loads page_content from
			  file <username>.<page_name>.html
*******************************************************************/
char *CWebBrowser::LoadPage(const char* page_name)
{
	CStringEx filename;
	char* page_content;
	filename.Format("%s/%s.%s.html", logdirenv ? logdirenv : DEFLOGDIR, username, page_name);
	FILE *fp = fopen(LPCTSTR(filename), "r");

	if(fp != NULL)
	{
		struct stat statbuf;
		stat(LPCTSTR(filename), &statbuf);
		page_content = (char *)malloc(statbuf.st_size+1);
		fread(page_content, 1, statbuf.st_size, fp);
		page_content[statbuf.st_size] = '\0';
		fclose(fp);               
		logFile.Write(LOG_ADVANCED, "<%s> Page has been loaded from '%s'", LPCTSTR(username), LPCTSTR(filename));
		return page_content;
	}
	else
	{
		logFile.Write(LOG_ADVANCED, "<%s> LoadPage: file '%s' could not be accessed.", LPCTSTR(username), LPCTSTR(filename));
		return NULL;
	}
}

/*******************************************************************
* Function  : DeleteEmails()
* Parameters: -
* Returns   : 0 if all communication succeeded
* Purpose   : To delete all messages that have been marked as 
*             deleted by the DELE command during the session.
*******************************************************************/
int CWebBrowser::DeleteEmail(MESSAGE *msg)
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	Email			   *email = &(msg->email);
	CStringEx			delUrl;
	CStringEx			folder = msg->mailbox;
	CStringEx			subj = email->GetHeader("Subject");

	delUrl.Format(yahoo[DEL], LPCTSTR(host), LPCTSTR(UrlEncode(folder)));
	delUrl += "&Mid=";
	delUrl += msg->msgId;
	/* Add the crumb */
	delUrl += "&.crumb=";
	delUrl += crumbValue;
	logFile.Write(LOG_ADVANCED, "<%s> Deleting msgId %s (%s) from Yahoo %s folder",
		LPCTSTR(username), LPCTSTR(msg->msgId), LPCTSTR(subj), LPCTSTR(folder));
	logFile.Write(LOG_ADVANCED, "<%s> DELETE URL: %s (length=%d)",
		LPCTSTR(username), LPCTSTR(delUrl), delUrl.length());
	c.Reset();
	resp.Reset();
	if (c.DoGet(delUrl, resp) != 0)
		return FALSE;
	return TRUE;
}

/*******************************************************************
* Function  : DeleteEmails()
* Parameters: -
* Returns   : 0 if all communication succeeded
* Purpose   : To delete all messages that have been marked as 
*             deleted by the DELE command during the session.
*******************************************************************/
int CWebBrowser::DeleteEmails(const CStdString &folder)
{
	HttpClient			c(session);
	HttpClientResponse	resp;
	CStringEx			delUrl;
	CStringEx			delData = ".crumb=" + crumbValue;
	int					count = 0;

	/* Delete all the emails now. */
	MESSAGE	*ptr = msgHead;
	
	while(ptr != NULL)
	{
		if(ptr->email.IsDeleted() && ptr->mailbox == folder)
		{
			count++;
			CStringEx subj = ptr->email.GetHeader("Subject");
			delData += "&Mid=" + ptr->msgId;
			logFile.Write(LOG_ADVANCED, "<%s> Deleting msgId %s (%s) from Yahoo %s folder",
				LPCTSTR(username), LPCTSTR(ptr->msgId), LPCTSTR(subj), LPCTSTR(folder));
		}
		
		ptr = ptr->next;
	}

	if (count > 0)
	{
		delUrl.Format(yahoo[DEL], LPCTSTR(host), LPCTSTR(UrlEncode(folder)));
		logFile.Write(LOG_ADVANCED, "<%s> DELETE URL: %s (length=%d)",
			LPCTSTR(username), LPCTSTR(delUrl), delUrl.length());
		c.Reset();
		resp.Reset();
		if (c.DoPost(delUrl, LPCTSTR(delData), resp) != 0)
			return 1;
		DumpPage(folder+".deleteresponse", resp.GetBodyPtr());
		return CheckError(folder+".deleteresponse", resp);
	}
	
	return 0; // nothing to do
}

int CWebBrowser::CheckError(const char *file, HttpClientResponse &resp)
{
	if (resp.DataContains("<div class=\"alertbox error\">"))
	{
		/* Extract IMG */
		CStringEx div;
		// If IMG extraction fails, bail out
		if (!div.split(resp.GetBodyPtr(), "<div class=\"alertbox error\">", "</div>"))
		{
			logFile.Write(LOG_BASIC, "<%s> Could not analyze error response page (\"alertbox error\" not found)", LPCTSTR(username));
			return 1;
		}

		logFile.Write(LOG_BASIC, 
			"<%s> Yahoo reported error: %s", LPCTSTR(username), LPCTSTR(div));
#ifdef WIN32
		CStringEx logdir = GetLogDir();

		CStringEx responsefile;
		responsefile.Format("file://%s/%s.%s.html", LPCTSTR(logdir), LPCTSTR(username), LPCTSTR(file));
		CStringEx html;
		html.Format("\
			<html><body>%s\n\
			<p>See <a target=ypops_error_response \
						href=\"%s\">complete Yahoo error response</a> for more details.\n\
			</body></html>",
			LPCTSTR(div), LPCTSTR(responsefile));
		YahooError(html);
#endif
		return 2;
	}

	if (resp.DataContains(YAHOO_ERROR_SESSION_EXPIRED))
	{
		YahooError(YAHOO_ERROR_SESSION_EXPIRED);
		logFile.Write(LOG_BASIC, 
			"<%s> Yahoo reported error: %s", LPCTSTR(username),
			LPCTSTR(YAHOO_ERROR_SESSION_EXPIRED));
		return 2;
	}

	return 0;
}

void CWebBrowser::YahooError(CStringEx str)
{
	DumpPage("yahooerror", LPCTSTR(str));
#ifdef WIN32
	CYahooErrorDlg dlg2;
	dlg2.m_url.Format("file://%s/%s.yahooerror.html", LPCTSTR(GetLogDir()), LPCTSTR(username));
    if (!username.IsEmpty())
    {
        CString title;
        title.Format("%s - YPOPs! Yahoo Error", LPCTSTR(username));
        dlg2.setTitle(title);
    }
    else
    {
        dlg2.setTitle("YPOPs! Yahoo Error");
    }
	int dr = dlg2.DoModal();
#endif
}

CStringEx CWebBrowser::GetLogDir()
{
	CStringEx logdir = logdirenv ? logdirenv : DEFLOGDIR;
#ifdef WIN32
	if (logdir.Compare(".") == 0)
	{
	   // Get the current working directory:
		char wdir[_MAX_PATH];

		if (getcwd(wdir, _MAX_PATH-1) != NULL)
		{
			wdir[_MAX_PATH] = '\0';
			logdir =  CStringEx(wdir);
		}
	}
#endif
	return logdir;
}
