/************************************************************************
  Filename   : IniFile.CPP
  Version    : 0.51
  Author(s)  : Carsten Breuer
 --[ Description ]-------------------------------------------------------

  This file contains a complete interface to read and write ini files
  like windows do it. I have ported it to CPP.
  This class have a simple test routine at the end of this file. To make
  a executable file define INIFILE_TEST_THIS_FILE after the includes

  --[ History ] ----------------------------------------------------------
	
  0.10: Original file by Carsten Breuer. First beta version.
  0.20: Ported to a C++ class
  0.30: Linux "\n" instead "\r\n" added. Thanks to Dieter Engelbrecht
  0.40: Problem with WriteString fixed.
  0.50: Problem with file pointer solved
  0.51: We better do smaller steps now. I have reformated to tab4. Sorry 
        New function DeleteKey added. Thanks for the guy who post this.
		I have also renamed the pointer to be sonsinstent with IniFile.Cpp.

  PLEASE update GetVersion!

 --[ How to compile ]----------------------------------------------------

  This file was developed under DJGPP and Rhide. If you are familiar with
  Borland C++ 3.1, you will feel like at home ;-)).
  Both tools are free software.

  Downloads at: http://www.delorie.com/djgpp


 --[ Where to get help/information ]-------------------------------------

  The author   : C.Breuer@OpenWin.de

 --[ License ] ----------------------------------------------------------

  LGPL (Free for private and comercial use)

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU Library General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU Library General Public License for more details.

  You should have received a copy of the GNU Library General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

 ------------------------------------------------------------------------
  Copyright (c) 2000 Carsten Breuer
/************************************************************************/

#include "stdafx.h"

// Standard Lib
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#ifdef INIFILE_TEST_THIS_FILE
#include <conio.h>	// Only for test: Used :  kbhit
#endif
#include "vsscanf.h"
#include "inifile.h"

/*
 * .ini file comments beginning with ; cause trouble for UserAgent in ypops,
 * so we don't treat as .ini file comments.
 */
#define HANDLE_COMMENTS

/* Here we start with our helper functions */
#define FreeMem(ptr) { if (ptr != NULL) free(ptr); ptr = NULL; }
/*
void CIniFile::FreeMem (void *pPtr)
{
	if (pPtr != NULL) { free (pPtr); }
}
*/


/*=========================================================================
   CIniFile : The constructor
*========================================================================*/
CIniFile::CIniFile (void)
{
	m_pEntry      = NULL;
	m_pCurEntry   = NULL;
	m_result      = "";
	m_pIniFile    = NULL;
}

/*=========================================================================
   CIniFile : The destructor
*========================================================================*/
CIniFile::~CIniFile (void)
{
	CloseIniFile ();
}

/*=========================================================================
   CIniFile : GetVersion
   Info     : The version is BCD coded. It maintain the major version in
              the upper 8 bits and the minor in the lower.
              0x0120 means version 1.20
*========================================================================*/
UINT CIniFile::GetVersion (void)
{
	return 0x0030;
}



/*=========================================================================
   OpenIniFile
  -------------------------------------------------------------------------
   Job : Opens an ini file or creates a new one if the requested file
         doesnt exists.
/*========================================================================*/
UINT CIniFile::OpenIniFile (CCHR *pFileName)
{
	char   Str [512];
	char   *pStr;
	struct ENTRY *pEntry;

	FreeAllMem ();

	if (pFileName == NULL)                             { return FALSE; }
	if ((m_pIniFile = fopen (pFileName, "r")) == NULL) { return FALSE; }

	while (fgets (Str, sizeof(Str), m_pIniFile) != NULL)
	{
		pStr = strchr (Str, '\n');
		if (pStr != NULL) { *pStr = 0; }
		pEntry = MakeNewEntry ();
		if (pEntry == NULL) { CloseIniFile(); return FALSE; }

#ifdef INI_REMOVE_CR
		int Len = strlen(Str);
		if ( Len > 0  && Str[Len-1] == '\r' )
            Str[Len-1] = '\0';
#endif

		pEntry->pText = (char *)malloc (strlen (Str)+1);
		if (pEntry->pText == NULL)
		{
			CloseIniFile();
			return FALSE;
		}
		strcpy (pEntry->pText, Str);
#ifdef HANDLE_COMMENTS
		pStr = strchr (Str,';');
		if (pStr != NULL) { *pStr = 0; } /* Cut all comments */
#endif
		if ( (strstr (Str, "[") > 0) && (strstr (Str, "]") > 0) ) /* Is Section */
		{
			pEntry->Type = tpSECTION;
		}
		else
		{
			if (strstr (Str, "=") > 0)
				pEntry->Type = tpKEYVALUE;
			else
				pEntry->Type = tpCOMMENT;
		}
		m_pCurEntry = pEntry;
	}
	fclose (m_pIniFile);
	m_pIniFile = NULL;
	return TRUE;
}

/*=========================================================================
   CloseIniFile
  -------------------------------------------------------------------------
   Job : Closes the ini file without any modifications. If you want to
         write the file use WriteIniFile instead.
/*========================================================================*/
void CIniFile::CloseIniFile (void)
{
	FreeAllMem ();
	if (m_pIniFile != NULL)
	{
		fclose (m_pIniFile);
		m_pIniFile = NULL;
	}
}


/*=========================================================================
   WriteIniFile
  -------------------------------------------------------------------------
   Job : Writes the iniFile to the disk and close it. Frees all memory
         allocated by WriteIniFile;
/*========================================================================*/
bool CIniFile::WriteIniFile (CCHR *pFileName)
{
	struct ENTRY *pEntry = m_pEntry;
	if (m_pIniFile != NULL)
		fclose (m_pIniFile);
	if ((m_pIniFile = fopen (pFileName, "wb")) == NULL)
	{
		//FreeAllMem ();
		return FALSE;
	}

	while (pEntry != NULL)
	{
		if (pEntry->Type != tpNULL)
		{
#ifdef INI_REMOVE_CR
			fprintf (m_pIniFile, "%s\n", pEntry->pText);
#else
			fprintf (m_pIniFile, "%s\r\n", pEntry->pText);
#endif
		}
		pEntry = pEntry->pNext;
	}

	fclose (m_pIniFile);
	m_pIniFile = NULL;
	return TRUE;
}


/*=========================================================================
   WriteString : Writes a string to the ini file
*========================================================================*/
void CIniFile::WriteString (CCHR *pSection, CCHR *pKey, CCHR *pValue)
{
	EFIND List;
	CStringEx Str;
    CStringEx Val;

	if (ArePtrValid (pSection, pKey, pValue) == FALSE) { return; }

    Val = pValue;
#ifdef HANDLE_COMMENTS
    // ; at last pos indicates to delete old comment in case the new str has none
    // this allows UserAgent to be written as-is.
    int offset = Val.ReverseFind(';');
    if (offset+1 == Val.GetLength())
        Val.Delete(offset);
#endif

	if (FindKey  (pSection, pKey, &List) == TRUE)
	{
#ifdef HANDLE_COMMENTS
        if (offset == -1)
            Val += List.Comment;
#endif
    	Str.Format("%s=%s", pKey, Val.c_str());
		FreeMem (List.pKey->pText);
		List.pKey->pText = (char *)calloc (1, strlen (Str.c_str())+1);
		ASSERT(List.pKey->pText != NULL);
		strcpy (List.pKey->pText, Str.c_str());
	}
	else
	{
		if ((List.pSec != NULL) && (List.pKey == NULL)) /* section exist, Key not */
		{
			AddKey (List.pSec, pKey, Val.c_str());
		}
		else
		{
			AddSectionAndKey (pSection, pKey, Val.c_str());
		}
	}
}

/*=========================================================================
   WriteBool : Writes a boolean to the ini file
*========================================================================*/
void CIniFile::WriteBool (CCHR *pSection, CCHR *pKey, UINT Value)
{
	WriteString (pSection, pKey, Value ? "1" : "0");
}

/*=========================================================================
   WriteInt : Writes an integer to the ini file
*========================================================================*/
void CIniFile::WriteInt (CCHR *pSection, CCHR *pKey, int Value)
{
	CStringEx Val; /* 32bit maximum + sign + \0 */
	Val.Format("%d", Value);
	WriteString(pSection, pKey, Val);
}

/*=========================================================================
   WriteDouble : Writes a double to the ini file
*========================================================================*/
void CIniFile::WriteDouble (CCHR *pSection, CCHR *pKey, double Value)
{
	CStringEx Val; /* DDDDDDDDDDDDDDD+E308\0 */
	Val.Format("%1.10lE", Value);
	WriteString(pSection, pKey, Val);
}


/*=========================================================================
   ReadStr : Reads a string from the ini file
*========================================================================*/
CCHR *CIniFile::ReadStr (CCHR *pSection, CCHR *pKey, CCHR *pDefault)
{
	EFIND List;
	if (ArePtrValid (pSection, pKey, pDefault) == FALSE) { return pDefault; }
	if (FindKey  (pSection, pKey, &List) == TRUE)
	{
		m_result = List.ValText;
		return m_result.c_str();
	}
	return pDefault;
}

/*=========================================================================
   ReadString : Reads a string (including comment) from the ini file
*========================================================================*/
CCHR *CIniFile::ReadString (CCHR *pSection, CCHR *pKey, CCHR *pDefault)
{
	EFIND List;
	if (ArePtrValid (pSection, pKey, pDefault) == FALSE) { return pDefault; }
	if (FindKey  (pSection, pKey, &List) == TRUE)
	{
		m_result  = List.ValText;
        m_result += List.Comment;
		return m_result.c_str();
	}
	return pDefault;
}

/*=========================================================================
   ReadBool : Reads a boolean from the ini file
*========================================================================*/
UINT CIniFile::ReadBool (CCHR *pSection, CCHR *pKey, UINT Default)
{
    return (atoi (ReadString (pSection, pKey, Default ? "1" : "0"))
                ?1:0); /* Only allow 0 or 1 */
}

/*=========================================================================
   ReadInt : Reads a integer from the ini file
*========================================================================*/
int CIniFile::ReadInt (CCHR *pSection, CCHR *pKey, int Default)
{
	CStringEx Val;
	Val.Format("%d", Default);
	return (atoi (ReadString (pSection, pKey, Val.c_str())));
}

/*=========================================================================
   ReadDouble : Reads a double from the ini file
*========================================================================*/
double CIniFile::ReadDouble (CCHR *pSection, CCHR *pKey, double Default)
{
	double Val;
	m_result.Format("%1.10lE", Default);
	sscanf (ReadString (pSection, pKey, m_result.c_str()), "%lE", &Val);
	return Val;
}

/*=========================================================================
   DeleteKey : Deletes an entry from the ini file
*========================================================================*/
bool CIniFile::DeleteKey (CCHR *pSection, CCHR *pKey)
{
	EFIND         List;
	struct ENTRY *pPrev;
	struct ENTRY *pNext;

	if (FindKey (pSection, pKey, &List) == TRUE)
	{
		pPrev = List.pKey->pPrev;
		pNext = List.pKey->pNext;
		if (pPrev)
			pPrev->pNext=pNext;
		if (pNext)
			pNext->pPrev=pPrev;
		FreeMem (List.pKey->pText);
		FreeMem (List.pKey);
		return TRUE;
	}
	return FALSE;
}


void CIniFile::FreeAllMem (void)
{
	struct ENTRY *pEntry;
	struct ENTRY *pNextEntry;

	pEntry = m_pEntry;

	while (pEntry != NULL)
	{
		pNextEntry = pEntry->pNext;
		FreeMem (pEntry->pText); /* Frees the pointer if not NULL */
		FreeMem (pEntry);
		pEntry = pNextEntry;
	}
	m_pEntry    = NULL;
	m_pCurEntry = NULL;
}

struct ENTRY *CIniFile::FindSection (CCHR *pSection)
{
	CStringEx Sec;
	Sec.Format("[%s]", pSection);
	struct ENTRY *pEntry = m_pEntry; /* Get a pointer to the first Entry */
	while (pEntry != NULL)
	{
		if (pEntry->Type == tpSECTION)
		{
			if (Sec.CompareNoCase(pEntry->pText) == 0)
				return pEntry;
		}
		pEntry = pEntry->pNext;
	}
	return NULL;
}

bool CIniFile::FindKey  (CCHR *pSection, CCHR *pKey, EFIND *pList)
{
	struct ENTRY *pEntry;
    memset(pList, sizeof(*pList), 0);
	pList->pSec       = NULL;
	pList->pKey       = NULL;
	pList->KeyText    = "";
	pList->ValText    = "";
	pList->Comment    = "";
	pEntry = FindSection (pSection);
	if (pEntry == NULL) { return FALSE; }
	pList->pSec       = pEntry;
	pEntry = pEntry->pNext;
	if (pEntry == NULL) { return FALSE; }
	CStringEx Search = pKey;
	while (pEntry != NULL)
	{
	    pList->KeyText = "";
	    pList->Comment = "";

		if ((pEntry->Type == tpSECTION) || /* Stop after next section or EOF */
			(pEntry->Type == tpNULL   ))
		{
			return FALSE;
		}
		if (pEntry->Type == tpKEYVALUE)
		{
			CStringEx Text = pEntry->pText;
#ifdef HANDLE_COMMENTS
			int iCom = Text.Find(';');
			if (iCom >= 0)
			{
                for (; Text[iCom] == ' '; iCom--);
				pList->Comment = Text.Mid(iCom);
				Text.Delete(iCom, Text.GetLength());
			}
#endif
			int iVal = Text.Find('=');
			if (iVal >= 0)
			{
				pList->KeyText = Text.Mid(0, iVal);
				if (Search.CompareNoCase(pList->KeyText) == 0)
				{
				   pList->ValText = Text.Mid(iVal+1);
				   pList->pKey = pEntry;
				   return TRUE;
				}
            }
		}
		pEntry = pEntry->pNext;
	}
	return FALSE;
}

UINT CIniFile::AddItem (char Type, CCHR *pText)
{
	struct ENTRY *pEntry = MakeNewEntry ();
	if (pEntry == NULL) { return FALSE; }
	pEntry->Type = Type;
	pEntry->pText = (char*)calloc (1, strlen (pText) +1);
	if (pEntry->pText == NULL)
	{
		FreeMem (pEntry);
		return FALSE;
	}
	strcpy (pEntry->pText, pText);
	pEntry->pNext   = NULL;
	if (m_pCurEntry != NULL) { m_pCurEntry->pNext = pEntry; }
	m_pCurEntry    = pEntry;
	return TRUE;
}

bool CIniFile::AddItemAt (struct ENTRY *pEntryAt, char Mode, CCHR *pText)
{
	struct ENTRY *pNewEntry;
	if (pEntryAt == NULL)  { return FALSE; }
	pNewEntry = (struct ENTRY*) calloc (1, sizeof (ENTRY));
	if (pNewEntry == NULL) { return FALSE; }
	pNewEntry->pText = (char *) calloc (1, strlen (pText)+1);
	if (pNewEntry->pText == NULL)
	{
		free (pNewEntry);
		return FALSE;
	}
	strcpy (pNewEntry->pText, pText);
	if (pEntryAt->pNext == NULL) /* No following nodes. */
	{
		pEntryAt->pNext   = pNewEntry;
		pNewEntry->pNext  = NULL;
	}
	else
	{
		pNewEntry->pNext = pEntryAt->pNext;
		pEntryAt->pNext  = pNewEntry;
	}
	pNewEntry->pPrev = pEntryAt;
	pNewEntry->Type  = Mode;
	return TRUE;
}

bool CIniFile::AddSectionAndKey (CCHR *pSection, CCHR *pKey, CCHR *pValue)
{
	CStringEx Text;
	Text.Format("[%s]", pSection);
	if (AddItem (tpSECTION, Text) == FALSE) { return FALSE; }
	Text.Format("%s=%s", pKey, pValue);
	return AddItem (tpKEYVALUE, Text)? 1 : 0;
}

void CIniFile::AddKey (struct ENTRY *pSecEntry, CCHR *pKey, CCHR *pValue)
{
	CStringEx Text;
	Text.Format("%s=%s", pKey, pValue);
	AddItemAt (pSecEntry, tpKEYVALUE, Text);
}

struct ENTRY *CIniFile::MakeNewEntry (void)
{
	struct ENTRY *pEntry;
	pEntry = (struct ENTRY *)malloc (sizeof (ENTRY));
	if (pEntry == NULL)
	{
		//FreeAllMem ();
		return NULL;
	}
	pEntry->Type  = tpNULL;
	pEntry->pPrev = m_pCurEntry;
	pEntry->pNext = NULL;
	pEntry->pText = NULL;
	if (m_pEntry == NULL)
		m_pEntry = pEntry;
	if (m_pCurEntry != NULL)
		m_pCurEntry->pNext = pEntry;
	return pEntry;
}



#ifdef INIFILE_TEST_THIS_FILE
int main (void)
{
	CIniFile iFile;
	printf ("Hello World\n");
	iFile.OpenIniFile  ("Test.Ini");
	iFile.WriteString  ("Test", "Name", "Value");
	iFile.WriteString  ("Test", "Name", "OverWrittenValue");
	iFile.WriteString  ("Test", "Port", "COM1");
	iFile.WriteString  ("Test", "User", "James Brown jr.");
	iFile.WriteString  ("Configuration", "eDriver", "MBM2.VXD");
	iFile.WriteString  ("Configuration", "Wrap", "LPT.VXD");
	iFile.WriteInt     ("IO-Port", "Com", 2);
	iFile.WriteBool    ("IO-Port", "IsValid", 0);
	iFile.WriteDouble  ("TheMoney", "TheMoney", 67892.00241);
	iFile.WriteInt     ("Test"    , "ToDelete", 1234);
	iFile.WriteIniFile ("Test.Ini");
	printf ("Key ToDelete created. Check ini file. Any key to continue");
	while (!kbhit());
	iFile.OpenIniFile  ("Test.Ini");
	iFile.DeleteKey    ("Test"	  , "ToDelete");
	iFile.WriteIniFile ("Test.Ini");
	printf ("[Test] Name = %s\n", iFile.ReadString ("Test", "Name", "NotFound"));
	printf ("[Test] Port = %s\n", iFile.ReadString ("Test", "Port", "NotFound"));
	printf ("[Test] User = %s\n", iFile.ReadString ("Test", "User", "NotFound"));
	printf ("[Configuration] eDriver = %s\n", iFile.ReadString ("Configuration", "eDriver", "NotFound"));
	printf ("[Configuration] Wrap = %s\n", iFile.ReadString ("Configuration", "Wrap", "NotFound"));
	printf ("[IO-Port] Com = %d\n", iFile.ReadInt ("IO-Port", "Com", 0));
	printf ("[IO-Port] IsValid = %d\n", iFile.ReadBool ("IO-Port", "IsValid", 0));
	printf ("[TheMoney] TheMoney = %1.10lf\n", iFile.ReadDouble ("TheMoney", "TheMoney", 111));
	iFile.CloseIniFile ();
	return 0;
}
#endif

