/************************************************************************
   T h e   O p e n   W i n d o w s   P r o j e c t
 ------------------------------------------------------------------------
   Filename   : IniFile.h
   Author(s)  : Carsten Breuer
 ------------------------------------------------------------------------
 Copyright (c) 2000 by Carsten Breuer (CBreuer@openwin.de)
/************************************************************************/

#include "vsscanf.h"

#ifndef INIFILE_H
#define INIFILE_H

#define OWP_DONT_DEF_BOOL
//#define OWP_DONT_DEF_UINT
//#define OWP_DONT_DEF_FALSE
//#define OWP_DONT_DEF_TRUE

#ifndef OWP_DONT_DEF_FALSE
#ifndef FALSE
#define FALSE 0
#endif
#endif

#ifndef OWP_DONT_DEF_TRUE
#ifndef TRUE
#define TRUE 1
#endif
#endif


#ifndef OWP_DONT_DEF_BOOL
#ifndef BOOL
#define BOOL unsigned int
#endif
#endif

#ifndef OWP_DONT_DEF_CCHR
#ifndef CCHR
typedef const char CCHR;
#endif
#endif

#ifndef OWP_DONT_DEF_UINT
#ifndef UINT
typedef unsigned int UINT;
#endif
#endif

#ifndef WIN32 /* Remove CR, on unix systems. */
#define INI_REMOVE_CR
#endif

#define tpNULL       0
#define tpSECTION    1
#define tpKEYVALUE   2
#define tpCOMMENT    3

struct ENTRY
{
   char   Type;
   char  *pText;
   struct ENTRY *pPrev;
   struct ENTRY *pNext;
};

typedef struct
{
   struct ENTRY *pSec;
   struct ENTRY *pKey;
   CStringEx     KeyText;
   CStringEx     ValText;
   CStringEx     Comment;
} EFIND;

/* Macros */
#define ArePtrValid(Sec,Key,Val) ((Sec!=NULL)&&(Key!=NULL)&&(Val!=NULL))

class CIniFile
{
public:
            CIniFile    (void);
            ~CIniFile   (void);
    UINT    GetVersion  (void);
    UINT    OpenIniFile (CCHR *pFileName);
    UINT    ReadBool    (CCHR *pSection, CCHR *pKey, UINT   Default);
    int     ReadInt     (CCHR *pSection, CCHR *pKey, int    Default);
    double  ReadDouble  (CCHR *pSection, CCHR *pKey, double Default);
    CCHR   *ReadStr     (CCHR *pSection, CCHR *pKey, CCHR  *pDefault);
    CCHR   *ReadString  (CCHR *pSection, CCHR *pKey, CCHR  *pDefault);

    void    WriteBool   (CCHR *pSection, CCHR *pKey, UINT   Value);
    void    WriteInt    (CCHR *pSection, CCHR *pKey, int    Value);
    void    WriteDouble (CCHR *pSection, CCHR *pKey, double Value);
    void    WriteString (CCHR *pSection, CCHR *pKey, CCHR  *pValue);

    void    CloseIniFile ();
    bool    WriteIniFile (CCHR *pFileName);
	bool	DeleteKey (CCHR *pSection, CCHR *pKey);

protected:
	struct  ENTRY *m_pEntry;
	struct  ENTRY *m_pCurEntry;
	CStringEx m_result;
	FILE    *m_pIniFile;
	void    AddKey     (struct ENTRY *pEntry, CCHR *pKey, CCHR *pValue);
	UINT    AddItem    (char Type, CCHR *pText);
	bool    AddItemAt (struct ENTRY *pEntryAt, char Mode, CCHR *pText);
	//void    FreeMem    (void *pPtr);
	void    FreeAllMem (void);
	bool    FindKey    (CCHR *pSection, CCHR *pKey, EFIND *pList);
	bool    AddSectionAndKey (CCHR *pSection, CCHR *pKey, CCHR *pValue);
	struct  ENTRY *MakeNewEntry (void);
	struct  ENTRY *FindSection (CCHR *pSection);
private:
};

#endif

